# TestSwiftAtObjCIvars.py
#
# This source file is part of the Swift.org open source project
#
# Copyright (c) 2014 - 2016 Apple Inc. and the Swift project authors
# Licensed under Apache License v2.0 with Runtime Library Exception
#
# See https://swift.org/LICENSE.txt for license information
# See https://swift.org/CONTRIBUTORS.txt for the list of Swift project authors
#
# ------------------------------------------------------------------------------
"""
Check that we correctly find offsets for ivars of Swift @objc types
"""
import lldb
from lldbsuite.test.lldbtest import *
from lldbsuite.test.decorators import *
import lldbsuite.test.lldbutil as lldbutil
import os
import unittest2


class TestSwiftAtObjCIvars(TestBase):
    def check_foo(self, theFoo):
        x = theFoo.GetChildMemberWithName("x")
        y = theFoo.GetChildMemberWithName("y")
        lldbutil.check_variable(self, x, False, value='12')
        lldbutil.check_variable(self, y, False, '"12"')

    @skipUnlessDarwin
    @swiftTest
    def test_swift_at_objc_ivars(self):
        """Test ObjC instance variables"""
        self.build()
        lldbutil.run_to_source_breakpoint(
            self, 'Set a breakpoint here', lldb.SBFileSpec('main.swift'))

        foo_objc = self.frame().FindVariable("foo_objc")
        foo_swift = self.frame().FindVariable("foo_swift")

        self.check_foo(foo_objc)
        self.check_foo(foo_swift)



# RUN: env SUPPORT_LIB=%mlir_c_runner_utils \
# RUN:   %PYTHON %s | FileCheck %s

import ctypes
import errno
import itertools
import os
import sys

from typing import List, Callable

import numpy as np

from mlir import ir
from mlir import runtime as rt

from mlir.dialects import bufferization
from mlir.dialects import builtin
from mlir.dialects import func
from mlir.dialects import sparse_tensor as st

_SCRIPT_PATH = os.path.dirname(os.path.abspath(__file__))
sys.path.append(_SCRIPT_PATH)
from tools import sparse_compiler

# ===----------------------------------------------------------------------=== #

# TODO: move this boilerplate to its own module, so it can be used by
# other tests and programs.
class TypeConverter:
    """Converter between NumPy types and MLIR types."""

    def __init__(self, context: ir.Context):
        # Note 1: these are numpy "scalar types" (i.e., the values of
        # np.sctypeDict) not numpy "dtypes" (i.e., the np.dtype class).
        #
        # Note 2: we must construct the MLIR types in the same context as the
        # types that'll be passed to irtype_to_sctype() or irtype_to_dtype();
        # otherwise, those methods will raise a KeyError.
        types_list = [
            (np.float64, ir.F64Type.get(context=context)),
            (np.float32, ir.F32Type.get(context=context)),
            (np.int64, ir.IntegerType.get_signless(64, context=context)),
            (np.int32, ir.IntegerType.get_signless(32, context=context)),
            (np.int16, ir.IntegerType.get_signless(16, context=context)),
            (np.int8, ir.IntegerType.get_signless(8, context=context)),
        ]
        self._sc2ir = dict(types_list)
        self._ir2sc = dict(((ir, sc) for sc, ir in types_list))

    def dtype_to_irtype(self, dtype: np.dtype) -> ir.Type:
        """Returns the MLIR equivalent of a NumPy dtype."""
        try:
            return self.sctype_to_irtype(dtype.type)
        except KeyError as e:
            raise KeyError(f"Unknown dtype: {dtype}") from e

    def sctype_to_irtype(self, sctype) -> ir.Type:
        """Returns the MLIR equivalent of a NumPy scalar type."""
        if sctype in self._sc2ir:
            return self._sc2ir[sctype]
        else:
            raise KeyError(f"Unknown sctype: {sctype}")

    def irtype_to_dtype(self, tp: ir.Type) -> np.dtype:
        """Returns the NumPy dtype equivalent of an MLIR type."""
        return np.dtype(self.irtype_to_sctype(tp))

    def irtype_to_sctype(self, tp: ir.Type):
        """Returns the NumPy scalar-type equivalent of an MLIR type."""
        if tp in self._ir2sc:
            return self._ir2sc[tp]
        else:
            raise KeyError(f"Unknown ir.Type: {tp}")

    def get_RankedTensorType_of_nparray(
        self, nparray: np.ndarray
    ) -> ir.RankedTensorType:
        """Returns the ir.RankedTensorType of a NumPy array.  Note that NumPy
        arrays can only be converted to/from dense tensors, not sparse tensors."""
        # TODO: handle strides as well?
        return ir.RankedTensorType.get(
            nparray.shape, self.dtype_to_irtype(nparray.dtype)
        )


# ===----------------------------------------------------------------------=== #


class StressTest:
    def __init__(self, tyconv: TypeConverter):
        self._tyconv = tyconv
        self._roundtripTp = None
        self._module = None
        self._engine = None

    def _assertEqualsRoundtripTp(self, tp: ir.RankedTensorType):
        assert self._roundtripTp is not None, "StressTest: uninitialized roundtrip type"
        if tp != self._roundtripTp:
            raise AssertionError(
                f"Type is not equal to the roundtrip type.\n"
                f"\tExpected: {self._roundtripTp}\n"
                f"\tFound:    {tp}\n"
            )

    def build(self, types: List[ir.Type]):
        """Builds the ir.Module.  The module has only the @main function,
        which will convert the input through the list of types and then back
        to the initial type.  The roundtrip type must be a dense tensor."""
        assert self._module is None, "StressTest: must not call build() repeatedly"
        self._module = ir.Module.create()
        with ir.InsertionPoint(self._module.body):
            tp0 = types.pop(0)
            self._roundtripTp = tp0
            # TODO: assert dense? assert element type is recognised by the TypeConverter?
            types.append(tp0)
            funcTp = ir.FunctionType.get(inputs=[tp0], results=[tp0])
            funcOp = func.FuncOp(name="main", type=funcTp)
            funcOp.attributes["llvm.emit_c_interface"] = ir.UnitAttr.get()
            with ir.InsertionPoint(funcOp.add_entry_block()):
                arg0 = funcOp.entry_block.arguments[0]
                self._assertEqualsRoundtripTp(arg0.type)
                v = st.ConvertOp(types.pop(0), arg0)
                for tp in types:
                    w = st.ConvertOp(tp, v)
                    # Release intermediate tensors before they fall out of scope.
                    bufferization.DeallocTensorOp(v.result)
                    v = w
                self._assertEqualsRoundtripTp(v.result.type)
                func.ReturnOp(v)
        return self

    def writeTo(self, filename):
        """Write the ir.Module to the given file.  If the file already exists,
        then raises an error.  If the filename is None, then is a no-op."""
        assert (
            self._module is not None
        ), "StressTest: must call build() before writeTo()"
        if filename is None:
            # Silent no-op, for convenience.
            return self
        if os.path.exists(filename):
            raise FileExistsError(errno.EEXIST, os.strerror(errno.EEXIST), filename)
        with open(filename, "w") as f:
            f.write(str(self._module))
        return self

    def compile(self, compiler):
        """Compile the ir.Module."""
        assert (
            self._module is not None
        ), "StressTest: must call build() before compile()"
        assert self._engine is None, "StressTest: must not call compile() repeatedly"
        self._engine = compiler.compile_and_jit(self._module)
        return self

    def run(self, np_arg0: np.ndarray) -> np.ndarray:
        """Runs the test on the given numpy array, and returns the resulting
        numpy array."""
        assert self._engine is not None, "StressTest: must call compile() before run()"
        self._assertEqualsRoundtripTp(
            self._tyconv.get_RankedTensorType_of_nparray(np_arg0)
        )
        np_out = np.zeros(np_arg0.shape, dtype=np_arg0.dtype)
        self._assertEqualsRoundtripTp(
            self._tyconv.get_RankedTensorType_of_nparray(np_out)
        )
        mem_arg0 = ctypes.pointer(
            ctypes.pointer(rt.get_ranked_memref_descriptor(np_arg0))
        )
        mem_out = ctypes.pointer(
            ctypes.pointer(rt.get_ranked_memref_descriptor(np_out))
        )
        self._engine.invoke("main", mem_out, mem_arg0)
        return rt.ranked_memref_to_numpy(mem_out[0])


# ===----------------------------------------------------------------------=== #


def main():
    """
    USAGE: python3 test_stress.py [raw_module.mlir [compiled_module.mlir]]

    The environment variable SUPPORT_LIB must be set to point to the
    libmlir_c_runner_utils shared library.  There are two optional
    arguments, for debugging purposes.  The first argument specifies where
    to write out the raw/generated ir.Module.  The second argument specifies
    where to write out the compiled version of that ir.Module.
    """
    support_lib = os.getenv("SUPPORT_LIB")
    assert support_lib is not None, "SUPPORT_LIB is undefined"
    if not os.path.exists(support_lib):
        raise FileNotFoundError(errno.ENOENT, os.strerror(errno.ENOENT), support_lib)

    # CHECK-LABEL: TEST: test_stress
    print("\nTEST: test_stress")
    with ir.Context() as ctx, ir.Location.unknown():
        # Disable direct sparse2sparse conversion, because it doubles the time!
        # TODO: While direct s2s is far too slow for per-commit testing,
        # we should have some framework ensure that we run this test with
        # `s2s=0` on a regular basis, to ensure that it does continue to work.
        # TODO: be sure to test s2s=0 together with singletons.
        s2s = 1
        sparsification_options = f"parallelization-strategy=none " f"s2s-strategy={s2s}"
        compiler = sparse_compiler.SparseCompiler(
            options=sparsification_options, opt_level=0, shared_libs=[support_lib]
        )
        f64 = ir.F64Type.get()
        # Be careful about increasing this because
        #     len(types) = 1 + len(level_choices)^rank * rank! * len(bitwidths)^2
        shape = range(2, 3)
        rank = len(shape)
        # All combinations.
        # TODO: add singleton here too; which requires updating how `np_arg0`
        # is initialized below.
        levels = list(
            itertools.product(
                *itertools.repeat(
                    [st.DimLevelType.dense, st.DimLevelType.compressed], rank
                )
            )
        )
        # All permutations.
        orderings = list(
            map(ir.AffineMap.get_permutation, itertools.permutations(range(rank)))
        )
        bitwidths = [0]
        # The first type must be a dense tensor for numpy conversion to work.
        types = [ir.RankedTensorType.get(shape, f64)]
        for level in levels:
            for ordering in orderings:
                for pwidth in bitwidths:
                    for iwidth in bitwidths:
                        attr = st.EncodingAttr.get(
                            level, ordering, pwidth, iwidth
                        )
                        types.append(ir.RankedTensorType.get(shape, f64, attr))
        #
        # For exhaustiveness we should have one or more StressTest, such
        # that their paths cover all 2*n*(n-1) directed pairwise combinations
        # of the `types` set.  However, since n is already superexponential,
        # such exhaustiveness would be prohibitive for a test that runs on
        # every commit.  So for now we'll just pick one particular path that
        # at least hits all n elements of the `types` set.
        #
        tyconv = TypeConverter(ctx)
        size = 1
        for d in shape:
            size *= d
        np_arg0 = np.arange(size, dtype=tyconv.irtype_to_dtype(f64)).reshape(*shape)
        np_out = (
            StressTest(tyconv)
            .build(types)
            .writeTo(sys.argv[1] if len(sys.argv) > 1 else None)
            .compile(compiler)
            .writeTo(sys.argv[2] if len(sys.argv) > 2 else None)
            .run(np_arg0)
        )
        # CHECK: Passed
        if np.allclose(np_out, np_arg0):
            print("Passed")
        else:
            sys.exit("FAILURE")


if __name__ == "__main__":
    main()



# TestLibraryResilient.py
#
# This source file is part of the Swift.org open source project
#
# Copyright (c) 2019 Apple Inc. and the Swift project authors
# Licensed under Apache License v2.0 with Runtime Library Exception
#
# See https://swift.org/LICENSE.txt for license information
# See https://swift.org/CONTRIBUTORS.txt for the list of Swift project authors
#
# ------------------------------------------------------------------------------
"""
Test `@_implementationOnly import` in a resilient library used by the main executable
"""
import lldb
from lldbsuite.test.lldbtest import *
from lldbsuite.test.decorators import *
import lldbsuite.test.lldbutil as lldbutil
import os
import os.path
import time
import unittest2

class TestLibraryResilient(TestBase):
    def launch_info(self):
        info = lldb.SBLaunchInfo([])
        info.SetLaunchFlags(lldb.eLaunchFlagInheritTCCFromParent)

        if self.getPlatform() == "freebsd" or self.getPlatform() == "linux":
            # LD_LIBRARY_PATH must be set so the shared libraries are found on
            # startup
            library_path = os.environ.get("LD_LIBRARY_PATH", "")
            if library_path:
                library_path += ":"
            library_path += self.getBuildDir()

            info.SetEnvironmentEntries(["LD_LIBRARY_PATH=" + library_path], True)

        return info

    @swiftTest
    @expectedFailureAll(oslist=["windows"]) # Requires Remote Mirrors support
    def test_implementation_only_import_library(self):
        """Test `@_implementationOnly import` in a resilient library used by the main executable

        See the ReadMe.md in the parent directory for more information.
        """

        self.build()
        lldbutil.run_to_source_breakpoint(self, "break here", lldb.SBFileSpec("main.swift"), self.launch_info())

        # This test is deliberately checking what the user will see, rather than
        # the structure provided by the Python API, in order to test the recovery.
        self.expect("fr var", substrs=[
            "(SomeLibrary.ContainsTwoInts) container = {\n  wrapped = (first = 2, second = 3)\n  other = 10\n}",
            "(Int) simple = 1"])
        self.expect("e container", substrs=["(SomeLibrary.ContainsTwoInts)", "other = 10"])
        self.expect("e container.wrapped", substrs=["(SomeLibraryCore.TwoInts)", "(first = 2, second = 3)"])

    @swiftTest
    @expectedFailureAll(oslist=["windows"]) # Requires Remote Mirrors support
    def test_implementation_only_import_library_no_library_module(self):
        """Test `@_implementationOnly import` in a resilient library used by the main executable, after removing the implementation-only library's swiftmodule

        See the ReadMe.md in the parent directory for more information.
        """

        self.build()
        os.remove(self.getBuildArtifact("SomeLibraryCore.swiftmodule"))
        os.remove(self.getBuildArtifact("SomeLibraryCore.swiftinterface"))
        lldbutil.run_to_source_breakpoint(self, "break here", lldb.SBFileSpec("main.swift"), self.launch_info())

        # This test is deliberately checking what the user will see, rather than
        # the structure provided by the Python API, in order to test the recovery.
        self.expect("fr var", substrs=[
            "(SomeLibrary.ContainsTwoInts) container = {", "other = 10",
            "(Int) simple = 1"])
        self.expect("e container", substrs=["(SomeLibrary.ContainsTwoInts)", "other = 10"])
        self.expect("e container.wrapped", error=True, substrs=["value of type 'ContainsTwoInts' has no member 'wrapped'"])



# Part of the LLVM Project, under the Apache License v2.0 with LLVM Exceptions.
# See https://llvm.org/LICENSE.txt for license information.
# SPDX-License-Identifier: Apache-2.0 WITH LLVM-exception

from subprocess import Popen
import os
import subprocess
import tempfile
import traceback
from ipykernel.kernelbase import Kernel

__version__ = "0.0.1"


def _get_executable():
    """Find the mlir-opt executable."""

    def is_exe(fpath):
        """Returns whether executable file."""
        return os.path.isfile(fpath) and os.access(fpath, os.X_OK)

    program = os.environ.get("MLIR_OPT_EXECUTABLE", "mlir-opt")
    path, name = os.path.split(program)
    # Attempt to get the executable
    if path:
        if is_exe(program):
            return program
    else:
        for path in os.environ["PATH"].split(os.pathsep):
            file = os.path.join(path, name)
            if is_exe(file):
                return file
    raise OSError("mlir-opt not found, please see README")


class MlirOptKernel(Kernel):
    """Kernel using mlir-opt inside jupyter.

    The reproducer syntax (`// configuration:`) is used to run passes. The
    previous result can be referenced to by using `_` (this variable is reset
    upon error). E.g.,

    ```mlir
    // configuration: --pass
    func.func @foo(%tensor: tensor<2x3xf64>) -> tensor<3x2xf64> { ... }
    ```

    ```mlir
    // configuration: --next-pass
    _
    ```
    """

    implementation = "mlir"
    implementation_version = __version__

    language_version = __version__
    language = "mlir"
    language_info = {
        "name": "mlir",
        "codemirror_mode": {"name": "mlir"},
        "mimetype": "text/x-mlir",
        "file_extension": ".mlir",
        "pygments_lexer": "text",
    }

    @property
    def banner(self):
        """Returns kernel banner."""
        # Just a placeholder.
        return "mlir-opt kernel %s" % __version__

    def __init__(self, **kwargs):
        Kernel.__init__(self, **kwargs)
        self._ = None
        self.executable = None
        self.silent = False

    def get_executable(self):
        """Returns the mlir-opt executable path."""
        if not self.executable:
            self.executable = _get_executable()
        return self.executable

    def process_output(self, output):
        """Reports regular command output."""
        if not self.silent:
            # Send standard output
            stream_content = {"name": "stdout", "text": output}
            self.send_response(self.iopub_socket, "stream", stream_content)

    def process_error(self, output):
        """Reports error response."""
        if not self.silent:
            # Send standard error
            stream_content = {"name": "stderr", "text": output}
            self.send_response(self.iopub_socket, "stream", stream_content)

    def do_execute(
        self, code, silent, store_history=True, user_expressions=None, allow_stdin=False
    ):
        """Execute user code using mlir-opt binary."""

        def ok_status():
            """Returns OK status."""
            return {
                "status": "ok",
                "execution_count": self.execution_count,
                "payload": [],
                "user_expressions": {},
            }

        def run(code):
            """Run the code by pipeing via filesystem."""
            try:
                inputmlir = tempfile.NamedTemporaryFile(delete=False)
                command = [
                    # Specify input and output file to error out if also
                    # set as arg.
                    self.get_executable(),
                    "--color",
                    inputmlir.name,
                    "-o",
                    "-",
                ]
                # Simple handling of repeating last line.
                if code.endswith("\n_"):
                    if not self._:
                        raise NameError("No previous result set")
                    code = code[:-1] + self._
                inputmlir.write(code.encode("utf-8"))
                inputmlir.close()
                pipe = Popen(command, stdout=subprocess.PIPE, stderr=subprocess.PIPE)
                output, errors = pipe.communicate()
                exitcode = pipe.returncode
            finally:
                os.unlink(inputmlir.name)

            # Replace temporary filename with placeholder. This takes the very
            # remote chance where the full input filename (generated above)
            # overlaps with something in the dump unrelated to the file.
            fname = inputmlir.name.encode("utf-8")
            output = output.replace(fname, b"<<input>>")
            errors = errors.replace(fname, b"<<input>>")
            return output, errors, exitcode

        self.silent = silent
        if not code.strip():
            return ok_status()

        try:
            output, errors, exitcode = run(code)

            if exitcode:
                self._ = None
            else:
                self._ = output.decode("utf-8")
        except KeyboardInterrupt:
            return {"status": "abort", "execution_count": self.execution_count}
        except Exception as error:
            # Print traceback for local debugging.
            traceback.print_exc()
            self._ = None
            exitcode = 255
            errors = repr(error).encode("utf-8")

        if exitcode:
            content = {"ename": "", "evalue": str(exitcode), "traceback": []}

            self.send_response(self.iopub_socket, "error", content)
            self.process_error(errors.decode("utf-8"))

            content["execution_count"] = self.execution_count
            content["status"] = "error"
            return content

        if not silent:
            data = {}
            data["text/x-mlir"] = self._
            content = {
                "execution_count": self.execution_count,
                "data": data,
                "metadata": {},
            }
            self.send_response(self.iopub_socket, "execute_result", content)
            self.process_output(self._)
            self.process_error(errors.decode("utf-8"))
        return ok_status()



import lldb
import sys


class WelcomeCommand(object):
    def __init__(self, debugger, session_dict):
        pass

    def get_short_help(self):
        return "Just a docstring for Welcome\nA command that says hello to LLDB users"

    def __call__(self, debugger, args, exe_ctx, result):
        print("Hello " + args + ", welcome to LLDB", file=result)
        return None


class WelcomeCommand2(object):
    def __init__(self, debugger, session_dict):
        pass

    def get_short_help(self):
        return "A docstring for the second Welcome\nA command that says hello to LLDB users"

    def __call__(self, debugger, args, exe_ctx, result):
        print("Hello " + args + ", welcome again to LLDB", file=result)
        return None



import lldb
from lldbsuite.test.decorators import *
from lldbsuite.test.lldbtest import *
from lldbsuite.test import lldbutil


class ValueAPIVoidStarTestCase(TestBase):
    def test(self):
        self.build()

        target, process, thread, _ = lldbutil.run_to_source_breakpoint(
            self, "Break at this line", lldb.SBFileSpec("main.c")
        )
        frame = thread.GetFrameAtIndex(0)

        # Verify that the expression result for a void * behaves the same way as the
        # variable value.

        var_val = frame.FindVariable("void_ptr")
        self.assertSuccess(var_val.GetError(), "Var version made correctly")

        expr_val = frame.EvaluateExpression("void_ptr")
        self.assertSuccess(expr_val.GetError(), "Expr version succeeds")

        # The pointer values should be equal:
        self.assertEqual(var_val.unsigned, expr_val.unsigned, "Values are equal")

        # Both versions should have valid AddressOf, and they should be the same.

        val_addr_of = var_val.AddressOf()
        self.assertNotEqual(val_addr_of, lldb.LLDB_INVALID_ADDRESS, "Var addr of right")

        expr_addr_of = expr_val.AddressOf()
        self.assertNotEqual(
            expr_addr_of, lldb.LLDB_INVALID_ADDRESS, "Expr addr of right"
        )

        # The AddressOf values should also be equal.
        self.assertEqual(expr_addr_of.unsigned, val_addr_of.unsigned, "Addr of equal")



"""
Test _regexp-break command which uses regular expression matching to dispatch to other built in breakpoint commands.
"""


import os
import lldb
from lldbsuite.test.lldbtest import *
import lldbsuite.test.lldbutil as lldbutil


class RegexpBreakCommandTestCase(TestBase):
    def test(self):
        """Test _regexp-break command."""
        self.build()
        self.regexp_break_command()

    def setUp(self):
        # Call super's setUp().
        TestBase.setUp(self)
        # Find the line number to break inside main().
        self.source = "main.c"
        self.line = line_number(self.source, "// Set break point at this line.")

    def regexp_break_command(self):
        """Test the super consie "b" command, which is analias for _regexp-break."""
        exe = self.getBuildArtifact("a.out")
        self.runCmd("file " + exe, CURRENT_EXECUTABLE_SET)

        break_results = lldbutil.run_break_set_command(self, "b %d" % self.line)
        lldbutil.check_breakpoint_result(
            self,
            break_results,
            file_name="main.c",
            line_number=self.line,
            num_locations=1,
        )

        break_results = lldbutil.run_break_set_command(
            self, "b %s:%d" % (self.source, self.line)
        )
        lldbutil.check_breakpoint_result(
            self,
            break_results,
            file_name="main.c",
            line_number=self.line,
            num_locations=1,
        )

        # Check breakpoint with full file path.
        full_path = os.path.join(self.getSourceDir(), self.source)
        break_results = lldbutil.run_break_set_command(
            self, "b %s:%d" % (full_path, self.line)
        )
        lldbutil.check_breakpoint_result(
            self,
            break_results,
            file_name="main.c",
            line_number=self.line,
            num_locations=1,
        )

        self.runCmd("run", RUN_SUCCEEDED)

        # The stop reason of the thread should be breakpoint.
        self.expect(
            "thread list",
            STOPPED_DUE_TO_BREAKPOINT,
            substrs=["stopped", "stop reason = breakpoint"],
        )



"""
Test that variables of integer basic types are displayed correctly.
"""


import AbstractBase
import lldb
from lldbsuite.test.lldbtest import *

# rdar://problem/9649573
# Capture the lldb and gdb-remote log files for test failures when run
# with no "-w" option


class DebugIntegerTypesFailures(TestBase):
    def setUp(self):
        # Call super's setUp().
        TestBase.setUp(self)
        # If we're lucky, test_long_type_with_dsym fails.
        # Let's turn on logging just for that.
        try:
            if "test_long_type_with_dsym" in self.id():
                self.runCmd(
                    "log enable -n -f %s lldb commands event process state"
                    % os.environ["DEBUG_LLDB_LOG"]
                )
                self.runCmd(
                    "log enable -n -f %s gdb-remote packets process"
                    % os.environ["DEBUG_GDB_REMOTE_LOG"]
                )
        except:
            pass

    def tearDown(self):
        # If we're lucky, test_long_type_with_dsym fails.
        # Let's turn off logging just for that.
        if "test_long_type_with_dsym" in self.id():
            self.runCmd("log disable lldb")
            self.runCmd("log disable gdb-remote")
        # Call super's tearDown().
        TestBase.tearDown(self)

    def test_char_type(self):
        """Test that char-type variables are displayed correctly."""
        d = {"CXX_SOURCES": "char.cpp"}
        self.build(dictionary=d)
        self.setTearDownCleanup(dictionary=d)
        self.generic_type_tester(set(["char"]), quotedDisplay=True)

    def test_short_type(self):
        """Test that short-type variables are displayed correctly."""
        d = {"CXX_SOURCES": "short.cpp"}
        self.build(dictionary=d)
        self.setTearDownCleanup(dictionary=d)
        self.generic_type_tester(set(["short"]))

    def test_int_type(self):
        """Test that int-type variables are displayed correctly."""
        d = {"CXX_SOURCES": "int.cpp"}
        self.build(dictionary=d)
        self.setTearDownCleanup(dictionary=d)
        self.generic_type_tester(set(["int"]))

    def test_long_type(self):
        """Test that long-type variables are displayed correctly."""
        d = {"CXX_SOURCES": "long.cpp"}
        self.build(dictionary=d)
        self.setTearDownCleanup(dictionary=d)
        self.generic_type_tester(set(["long"]))

    def test_long_long_type(self):
        """Test that 'long long'-type variables are displayed correctly."""
        d = {"CXX_SOURCES": "long_long.cpp"}
        self.build(dictionary=d)
        self.setTearDownCleanup(dictionary=d)
        self.generic_type_tester(set(["long long"]))



"""
Test that we read the function starts section.
"""


import lldb
from lldbsuite.test.decorators import *
from lldbsuite.test.lldbtest import *
from lldbsuite.test import lldbutil

exe_name = "StripMe"  # Must match Makefile


class FunctionStartsTestCase(TestBase):
    NO_DEBUG_INFO_TESTCASE = True

    @skipIfRemote
    @skipUnlessDarwin
    @skipIf(compiler="clang", compiler_version=["<", "13.0"])
    def test_function_starts_binary(self):
        """Test that we make synthetic symbols when we have the binary."""
        self.build(dictionary={"CODESIGN": ""})  # Binary is getting stripped later.
        self.do_function_starts(False)

    @skipIfRemote
    @skipUnlessDarwin
    @skipIf(compiler="clang", compiler_version=["<", "13.0"])
    def test_function_starts_no_binary(self):
        """Test that we make synthetic symbols when we don't have the binary"""
        self.build(dictionary={"CODESIGN": ""})  # Binary is getting stripped later.
        self.do_function_starts(True)

    def do_function_starts(self, in_memory):
        """Run the binary, stop at our unstripped function,
        make sure the caller has synthetic symbols"""

        exe = os.path.realpath(self.getBuildArtifact(exe_name))
        # Now strip the binary, but leave externals so we can break on dont_strip_me.
        self.runBuildCommand(["strip", "-u", "-x", "-S", exe])

        # Use a file as a synchronization point between test and inferior.
        pid_file_path = lldbutil.append_to_process_working_directory(
            self, "token_pid_%d" % (int(os.getpid()))
        )
        self.addTearDownHook(
            lambda: self.run_platform_command("rm %s" % (pid_file_path))
        )

        popen = self.spawnSubprocess(exe, [pid_file_path])

        # Wait until process has fully started up.
        pid = lldbutil.wait_for_file_on_target(self, pid_file_path)

        if in_memory:
            remove_file(exe)

        target = self.dbg.CreateTarget(None)
        self.assertTrue(target.IsValid(), "Got a vaid empty target.")
        error = lldb.SBError()
        attach_info = lldb.SBAttachInfo()
        attach_info.SetProcessID(popen.pid)
        attach_info.SetIgnoreExisting(False)
        process = target.Attach(attach_info, error)
        self.assertSuccess(error, "Didn't attach successfully to %d" % (popen.pid))

        bkpt = target.BreakpointCreateByName("dont_strip_me", exe)
        self.assertTrue(
            bkpt.GetNumLocations() > 0, "Didn't set the dont_strip_me bkpt."
        )

        threads = lldbutil.continue_to_breakpoint(process, bkpt)
        self.assertEqual(len(threads), 1, "Didn't hit my breakpoint.")

        # Our caller frame should have been stripped.  Make sure we made a synthetic symbol
        # for it:
        thread = threads[0]
        self.assertTrue(thread.num_frames > 1, "Couldn't backtrace.")
        name = thread.frame[1].GetFunctionName()
        self.assertTrue(name.startswith("___lldb_unnamed_symbol"))



import lldb
from lldbsuite.test.lldbtest import *
from lldbsuite.test.decorators import *
from lldbsuite.test.gdbclientutils import *
from lldbsuite.test.lldbgdbclient import GDBRemoteTestBase


class TestPlatformMacOSX(GDBRemoteTestBase):
    class MyResponder(MockGDBServerResponder):
        def __init__(self, host):
            self.host_ostype = host
            MockGDBServerResponder.__init__(self)

        def respond(self, packet):
            if packet == "qProcessInfo":
                return self.qProcessInfo()
            return MockGDBServerResponder.respond(self, packet)

        def qHostInfo(self):
            return (
                "cputype:16777223;cpusubtype:2;ostype:%s;vendor:apple;os_version:10.15.4;maccatalyst_version:13.4;endian:little;ptrsize:8;"
                % self.host_ostype
            )

        def qProcessInfo(self):
            return "pid:a860;parent-pid:d2a0;real-uid:1f5;real-gid:14;effective-uid:1f5;effective-gid:14;cputype:100000c;cpusubtype:2;ptrsize:8;ostype:ios;vendor:apple;endian:little;"

        def vCont(self):
            return "vCont;"

    def platform_test(self, host, expected_triple, expected_platform):
        self.server.responder = self.MyResponder(host)
        if self.TraceOn():
            self.runCmd("log enable gdb-remote packets")
            self.addTearDownHook(lambda: self.runCmd("log disable gdb-remote packets"))

        target = self.dbg.CreateTargetWithFileAndArch(None, None)
        process = self.connect(target)

        triple = target.GetTriple()
        self.assertEqual(triple, expected_triple)

        platform = target.GetPlatform()
        self.assertEqual(platform.GetName(), expected_platform)

    @skipIfRemote
    def test_ios(self):
        self.platform_test(
            host="ios",
            expected_triple="arm64e-apple-ios-",
            expected_platform="remote-ios",
        )

    @skipIfRemote
    @skipUnlessDarwin
    @skipUnlessArch("arm64")
    def test_macos(self):
        self.platform_test(
            host="macosx", expected_triple="arm64e-apple-ios-", expected_platform="host"
        )



# TestMainExecutable.py
#
# This source file is part of the Swift.org open source project
#
# Copyright (c) 2019 Apple Inc. and the Swift project authors
# Licensed under Apache License v2.0 with Runtime Library Exception
#
# See https://swift.org/LICENSE.txt for license information
# See https://swift.org/CONTRIBUTORS.txt for the list of Swift project authors
#
# ------------------------------------------------------------------------------
"""
Test `@_implementationOnly import` in the main executable
"""
import lldb
from lldbsuite.test.lldbtest import *
from lldbsuite.test.decorators import *
import lldbsuite.test.lldbutil as lldbutil
import os
import os.path
import time
import unittest2

@skipIfDarwin # rdar://problem/54322424 Sometimes failing, sometimes truncated output.
class TestMainExecutable(TestBase):
    def launch_info(self):
        info = lldb.SBLaunchInfo([])

        if self.getPlatform() == "freebsd" or self.getPlatform() == "linux":
            # LD_LIBRARY_PATH must be set so the shared libraries are found on
            # startup
            library_path = os.environ.get("LD_LIBRARY_PATH", "")
            if library_path:
                library_path += ":"
            library_path += self.getBuildDir()

            info.SetEnvironmentEntries(["LD_LIBRARY_PATH=" + library_path], True)

        return info

    @skipIf(bugnumber="rdar://problem/54322424", # This test is unreliable.
            setting=('symbols.use-swift-clangimporter', 'false'))
    @swiftTest
    def test_implementation_only_import_main_executable(self):
        """Test `@_implementationOnly import` in the main executable

        See the ReadMe.md in the parent directory for more information.
        """

        self.build()
        lldbutil.run_to_source_breakpoint(self, "break here", lldb.SBFileSpec("main.swift"), self.launch_info())

        # This test is deliberately checking what the user will see, rather than
        # the structure provided by the Python API, in order to test the recovery.
        self.expect("fr var", substrs=[
            "(SomeLibrary.TwoInts) value = (first = 2, second = 3)",
            "(main.ContainsTwoInts) container = {\n  wrapped = (first = 2, second = 3)\n  other = 10\n}",
            "(Int) simple = 1"])
        self.expect("e value", substrs=["(SomeLibrary.TwoInts)", "= (first = 2, second = 3)"])
        self.expect("e container", substrs=["(main.ContainsTwoInts)", "wrapped = (first = 2, second = 3)", "other = 10"])
        self.expect("e TwoInts(4, 5)", substrs=["(SomeLibrary.TwoInts)", "= (first = 4, second = 5)"])

    @skipIf(bugnumber="rdar://problem/54322424", # This test is unreliable.
            setting=('symbols.use-swift-clangimporter', 'false'))
    @swiftTest
    @skipIfLinux # rdar://problem/67348391
    def test_implementation_only_import_main_executable_no_library_module(self):
        """Test `@_implementationOnly import` in the main executable, after removing the library's swiftmodule

        See the ReadMe.md in the parent directory for more information.
        """

        self.build()
        self.runCmd("settings set symbols.use-swift-dwarfimporter false")
        os.remove(self.getBuildArtifact("SomeLibrary.swiftmodule"))
        os.remove(self.getBuildArtifact("SomeLibrary.swiftinterface"))
        lldbutil.run_to_source_breakpoint(self, "break here", lldb.SBFileSpec("main.swift"), self.launch_info())

        # FIXME: This particular test config is producing different results on
        # different machines, but it's also the least important configuration
        # (trying to debug something built against a library without that
        # library present, implementation-only or not, and that library doesn't
        # even have library evolution support enabled. Just make sure we don't
        # crash.
        self.expect("fr var", substrs=[
            "value = <could not resolve type>",
#            "container = {}",
            "simple = 1"])

        self.expect("e value", error=True)
        self.expect("e container", error=True)
        self.expect("e TwoInts(4, 5)", error=True)
        lldb.SBDebugger.MemoryPressureDetected()
        self.runCmd("settings set symbols.use-swift-dwarfimporter true")

    @swiftTest
    @expectedFailureAll(oslist=["windows"])
    def test_implementation_only_import_main_executable_resilient(self):
        """Test `@_implementationOnly import` in the main executable with a resilient library

        See the ReadMe.md in the parent directory for more information.
        """

        self.build(dictionary={"LIBRARY_SWIFTFLAGS_EXTRAS": "-enable-library-evolution"})
        lldbutil.run_to_source_breakpoint(self, "break here", lldb.SBFileSpec("main.swift"), self.launch_info())

        # This test is deliberately checking what the user will see, rather than
        # the structure provided by the Python API, in order to test the recovery.
        self.expect("fr var", substrs=[
            "(SomeLibrary.TwoInts) value = (first = 2, second = 3)",
            "(main.ContainsTwoInts) container = {\n  wrapped = (first = 2, second = 3)\n  other = 10\n}",
            "(Int) simple = 1"])
        self.expect("e value", substrs=["(SomeLibrary.TwoInts)", "= (first = 2, second = 3)"])
        self.expect("e container", substrs=["(main.ContainsTwoInts)", "wrapped = (first = 2, second = 3)", "other = 10"])
        self.expect("e TwoInts(4, 5)", substrs=["(SomeLibrary.TwoInts)", "= (first = 4, second = 5)"])

    @swiftTest
    @expectedFailureOS(no_match(["macosx"])) # Requires Remote Mirrors support
    def test_implementation_only_import_main_executable_resilient_no_library_module(self):
        """Test `@_implementationOnly import` in the main executable with a resilient library, after removing the library's swiftmodule

        See the ReadMe.md in the parent directory for more information.
        """

        self.build(dictionary={"LIBRARY_SWIFTFLAGS_EXTRAS": "-enable-library-evolution"})
        os.remove(self.getBuildArtifact("SomeLibrary.swiftmodule"))
        os.remove(self.getBuildArtifact("SomeLibrary.swiftinterface"))
        lldbutil.run_to_source_breakpoint(self, "break here", lldb.SBFileSpec("main.swift"), self.launch_info())

        # This test is deliberately checking what the user will see, rather than
        # the structure provided by the Python API, in order to test the recovery.
        self.expect("fr var", substrs=[
            "value = <could not resolve type>",
            "(main.ContainsTwoInts) container = (other = 10)",
            "(Int) simple = 1"])
        # FIXME: If we could figure out how to ignore this failure but still not
        # crash if we touch something that can't be loaded, that would be nice.
        self.expect("e value", error=True, substrs=["failed to get module \"SomeLibrary\" from AST context"])
        self.expect("e container", error=True, substrs=["failed to get module \"SomeLibrary\" from AST context"])
        self.expect("e TwoInts(4, 5)", error=True, substrs=["failed to get module \"SomeLibrary\" from AST context"])



# RUN: env SUPPORT_LIB=%mlir_c_runner_utils %PYTHON %s | FileCheck %s

import ctypes
import numpy as np
import os
import sys

from mlir import ir
from mlir import runtime as rt
from mlir.dialects import sparse_tensor as st
from mlir.dialects import builtin
from mlir.dialects.linalg.opdsl import lang as dsl

_SCRIPT_PATH = os.path.dirname(os.path.abspath(__file__))
sys.path.append(_SCRIPT_PATH)
from tools import np_to_sparse_tensor as test_tools
from tools import sparse_compiler

# TODO: Use linalg_structured_op to generate the kernel after making it to
# handle sparse tensor outputs.
_KERNEL_STR = """
#DCSR = #sparse_tensor.encoding<{
  lvlTypes = [ "compressed", "compressed" ]
}>

#trait_add_elt = {
  indexing_maps = [
    affine_map<(i,j) -> (i,j)>,  // A
    affine_map<(i,j) -> (i,j)>,  // B
    affine_map<(i,j) -> (i,j)>   // X (out)
  ],
  iterator_types = ["parallel", "parallel"],
  doc = "X(i,j) = A(i,j) + B(i,j)"
}

func.func @sparse_add_elt(
    %arga: tensor<3x4xf64, #DCSR>, %argb: tensor<3x4xf64, #DCSR>) -> tensor<3x4xf64, #DCSR> {
  %argx = bufferization.alloc_tensor() : tensor<3x4xf64, #DCSR>
  %0 = linalg.generic #trait_add_elt
    ins(%arga, %argb: tensor<3x4xf64, #DCSR>, tensor<3x4xf64, #DCSR>)
    outs(%argx: tensor<3x4xf64, #DCSR>) {
      ^bb(%a: f64, %b: f64, %x: f64):
        %1 = arith.addf %a, %b : f64
        linalg.yield %1 : f64
  } -> tensor<3x4xf64, #DCSR>
  return %0 : tensor<3x4xf64, #DCSR>
}

func.func @main(%ad: tensor<3x4xf64>, %bd: tensor<3x4xf64>) -> tensor<3x4xf64, #DCSR>
  attributes { llvm.emit_c_interface } {
  %a = sparse_tensor.convert %ad : tensor<3x4xf64> to tensor<3x4xf64, #DCSR>
  %b = sparse_tensor.convert %bd : tensor<3x4xf64> to tensor<3x4xf64, #DCSR>
  %0 = call @sparse_add_elt(%a, %b) : (tensor<3x4xf64, #DCSR>, tensor<3x4xf64, #DCSR>) -> tensor<3x4xf64, #DCSR>
  return %0 : tensor<3x4xf64, #DCSR>
}
"""


def _run_test(support_lib, kernel):
    """Compiles, runs and checks results."""
    compiler = sparse_compiler.SparseCompiler(
        options="", opt_level=2, shared_libs=[support_lib]
    )
    module = ir.Module.parse(kernel)
    engine = compiler.compile_and_jit(module)

    # Set up numpy inputs and buffer for output.
    a = np.array(
        [[1.1, 0.0, 0.0, 0.0], [0.0, 0.0, 0.0, 0.0], [0.0, 0.0, 6.6, 0.0]], np.float64
    )
    b = np.array(
        [[1.1, 0.0, 0.0, 2.8], [0.0, 0.0, 0.0, 0.0], [0.0, 0.0, 0.0, 0.0]], np.float64
    )

    mem_a = ctypes.pointer(ctypes.pointer(rt.get_ranked_memref_descriptor(a)))
    mem_b = ctypes.pointer(ctypes.pointer(rt.get_ranked_memref_descriptor(b)))

    # The sparse tensor output is a pointer to pointer of char.
    out = ctypes.c_char(0)
    mem_out = ctypes.pointer(ctypes.pointer(out))

    # Invoke the kernel.
    engine.invoke("main", mem_a, mem_b, mem_out)

    # Retrieve and check the result.
    rank, nse, shape, values, indices = test_tools.sparse_tensor_to_coo_tensor(
        support_lib, mem_out[0], np.float64
    )

    # CHECK: PASSED
    if np.allclose(values, [2.2, 2.8, 6.6]) and np.allclose(
        indices, [[0, 0], [0, 3], [2, 2]]
    ):
        print("PASSED")
    else:
        quit("FAILURE")


def test_elementwise_add():
    # Obtain path to runtime support library.
    support_lib = os.getenv("SUPPORT_LIB")
    assert support_lib is not None, "SUPPORT_LIB is undefined"
    assert os.path.exists(support_lib), f"{support_lib} does not exist"
    with ir.Context() as ctx, ir.Location.unknown():
        _run_test(support_lib, _KERNEL_STR)


test_elementwise_add()



import lldb
from lldbsuite.test.decorators import *
import lldbsuite.test.lldbtest as lldbtest
import lldbsuite.test.lldbutil as lldbutil


class TestCase(lldbtest.TestBase):
    @swiftTest
    @skipIf(oslist=["windows", "linux"])
    def test(self):
        """Test conditions for async step-in."""
        self.build()

        src = lldb.SBFileSpec("main.swift")
        target, _, thread, _ = lldbutil.run_to_source_breakpoint(self, "await f()", src)
        self.assertEqual(thread.frame[0].function.mangled, "$s1a5entryO4mainyyYaFZ")

        function = target.FindFunctions("$s1a5entryO4mainyyYaFZTQ0_")[0].function
        instructions = list(function.GetInstructions(target))
        self.assertGreater(len(instructions), 0)
        # Expected to be a trampoline that tail calls `swift_task_switch`.
        self.assertIn("swift_task_switch", instructions[-1].GetComment(target))

        # Using the line table, build a set of the non-zero line numbers for
        # this this function - and verify that there is exactly one line.
        lines = {inst.addr.line_entry.line for inst in instructions}
        lines.remove(0)
        self.assertEqual(lines, {3})

        # Required for builds that have debug info.
        self.runCmd("settings set target.process.thread.step-avoid-libraries libswift_Concurrency.dylib")
        thread.StepInto()
        frame = thread.frame[0]
        # Step in from `main` should progress through to `f`.
        self.assertEqual(frame.name, "a.f() async -> Swift.Int")



"""
Test that the lldb driver's batch mode works correctly.
"""


import lldb
from lldbsuite.test.decorators import *
from lldbsuite.test.lldbtest import *
from lldbsuite.test import lldbutil
from lldbsuite.test.lldbpexpect import PExpectTest


class DriverBatchModeTest(PExpectTest):
    source = "main.c"

    @skipIf(oslist=["linux"], archs=["arm", "aarch64"])  # Randomly fails on buildbot
    @expectedFlakeyFreeBSD("llvm.org/pr25172 fails rarely on the buildbot")
    def test_batch_mode_run_crash(self):
        """Test that the lldb driver's batch mode works correctly."""
        self.build()

        exe = self.getBuildArtifact("a.out")

        # Pass CRASH so the process will crash and stop in batch mode.
        extra_args = [
            "-b",
            "-o",
            "break set -n main",
            "-o",
            "run",
            "-o",
            "continue",
            "-k",
            "frame var touch_me_not",
            "--",
            "CRASH",
        ]
        self.launch(executable=exe, extra_args=extra_args)
        child = self.child

        # We should see the "run":
        child.expect_exact("run")
        # We should have hit the breakpoint & continued:
        child.expect_exact("continue")
        # The App should have crashed:
        child.expect_exact("About to crash")
        # The -k option should have printed the frame variable once:
        child.expect_exact("(char *) touch_me_not")
        # Then we should have a live prompt:
        self.expect_prompt()
        self.expect("frame variable touch_me_not", substrs=["(char *) touch_me_not"])

    @skipIf(oslist=["linux"], archs=["arm", "aarch64"])  # Randomly fails on buildbot
    @expectedFlakeyFreeBSD("llvm.org/pr25172 fails rarely on the buildbot")
    def test_batch_mode_run_exit(self):
        """Test that the lldb driver's batch mode works correctly."""
        self.build()

        exe = self.getBuildArtifact("a.out")

        # Now do it again, and make sure if we don't crash, we quit:
        extra_args = [
            "-b",
            "-o",
            "break set -n main",
            "-o",
            "run",
            "-o",
            "continue",
            "--",
            "NOCRASH",
        ]
        self.launch(executable=exe, extra_args=extra_args)
        child = self.child

        # We should see the "run":
        child.expect_exact("run")
        # We should have hit the breakpoint & continued:
        child.expect_exact("continue")
        # The App should have not have crashed:
        child.expect_exact("Got there on time and it did not crash.")

        # Then lldb should exit.
        child.expect_exact("exited")
        import pexpect

        child.expect(pexpect.EOF)

    @skipIf(oslist=["linux"], archs=["arm", "aarch64"])  # Randomly fails on buildbot
    @expectedFlakeyFreeBSD("llvm.org/pr25172 fails rarely on the buildbot")
    def test_batch_mode_launch_stop_at_entry(self):
        """Test that the lldb driver's batch mode works correctly for process launch."""
        self.build()

        exe = self.getBuildArtifact("a.out")

        # Launch with the option '--stop-at-entry' stops with a signal (usually SIGSTOP)
        # that should be suppressed since it doesn't imply a crash and
        # this is not a reason to exit batch mode.
        extra_args = [
            "-b",
            "-o",
            "process launch --stop-at-entry",
            "-o",
            "continue",
        ]
        self.launch(executable=exe, extra_args=extra_args)
        child = self.child

        # Check that the process has been launched:
        child.expect("Process ([0-9]+) launched:")
        # We should have continued:
        child.expect_exact("continue")
        # The App should have not have crashed:
        child.expect_exact("Got there on time and it did not crash.")

        # Then lldb should exit.
        child.expect_exact("exited")
        import pexpect

        child.expect(pexpect.EOF)

    def closeVictim(self):
        if self.victim is not None:
            self.victim.close()
            self.victim = None

    @skipIf(oslist=["linux"], archs=["arm", "aarch64"])  # Randomly fails on buildbot
    @expectedFlakeyFreeBSD("llvm.org/pr25172 fails rarely on the buildbot")
    @expectedFailureNetBSD
    def test_batch_mode_attach_exit(self):
        """Test that the lldb driver's batch mode works correctly."""
        self.build()
        self.setTearDownCleanup()

        exe = self.getBuildArtifact("a.out")

        # Start up the process by hand, attach to it, and wait for its completion.
        # Attach is funny, since it looks like it stops with a signal on most Unixen so
        # care must be taken not to treat that as a reason to exit batch mode.

        # Start up the process by hand and wait for it to get to the wait loop.
        import pexpect

        self.victim = pexpect.spawn("%s WAIT" % (exe))
        if self.victim is None:
            self.fail("Could not spawn ", exe, ".")

        self.addTearDownHook(self.closeVictim)

        self.victim.expect("PID: ([0-9]+) END")
        victim_pid = int(self.victim.match.group(1))

        self.victim.expect("Waiting")

        extra_args = [
            "-b",
            "-o",
            "process attach -p %d" % victim_pid,
            "-o",
            "breakpoint set --file '%s' -p 'Stop here to unset keep_waiting' -N keep_waiting"
            % self.source,
            "-o",
            "continue",
            "-o",
            "break delete keep_waiting",
            "-o",
            "expr keep_waiting = 0",
            "-o",
            "continue",
        ]
        self.launch(executable=exe, extra_args=extra_args)
        child = self.child

        child.expect_exact("attach")

        child.expect_exact(self.PROMPT + "continue")

        child.expect_exact(self.PROMPT + "continue")

        # Then we should see the process exit:
        child.expect_exact("Process %d exited with status" % (victim_pid))

        self.victim.expect(pexpect.EOF)
        child.expect(pexpect.EOF)



# RUN: %PYTHON %s | FileCheck %s

from mlir.ir import *
import mlir.dialects.builtin as builtin
import mlir.dialects.func as func
import numpy as np


def run(f):
    print("\nTEST:", f.__name__)
    f()
    return f


# CHECK-LABEL: TEST: testFromPyFunc
@run
def testFromPyFunc():
    with Context() as ctx, Location.unknown() as loc:
        ctx.allow_unregistered_dialects = True
        m = builtin.ModuleOp()
        f32 = F32Type.get()
        f64 = F64Type.get()
        with InsertionPoint(m.body):
            # CHECK-LABEL: func @unary_return(%arg0: f64) -> f64
            # CHECK: return %arg0 : f64
            @func.FuncOp.from_py_func(f64)
            def unary_return(a):
                return a

            # CHECK-LABEL: func @binary_return(%arg0: f32, %arg1: f64) -> (f32, f64)
            # CHECK: return %arg0, %arg1 : f32, f64
            @func.FuncOp.from_py_func(f32, f64)
            def binary_return(a, b):
                return a, b

            # CHECK-LABEL: func @none_return(%arg0: f32, %arg1: f64)
            # CHECK: return
            @func.FuncOp.from_py_func(f32, f64)
            def none_return(a, b):
                pass

            # CHECK-LABEL: func @call_unary
            # CHECK: %0 = call @unary_return(%arg0) : (f64) -> f64
            # CHECK: return %0 : f64
            @func.FuncOp.from_py_func(f64)
            def call_unary(a):
                return unary_return(a)

            # CHECK-LABEL: func @call_binary
            # CHECK: %0:2 = call @binary_return(%arg0, %arg1) : (f32, f64) -> (f32, f64)
            # CHECK: return %0#0, %0#1 : f32, f64
            @func.FuncOp.from_py_func(f32, f64)
            def call_binary(a, b):
                return binary_return(a, b)

            # We expect coercion of a single result operation to a returned value.
            # CHECK-LABEL: func @single_result_op
            # CHECK: %0 = "custom.op1"() : () -> f32
            # CHECK: return %0 : f32
            @func.FuncOp.from_py_func()
            def single_result_op():
                return Operation.create("custom.op1", results=[f32])

            # CHECK-LABEL: func @call_none
            # CHECK: call @none_return(%arg0, %arg1) : (f32, f64) -> ()
            # CHECK: return
            @func.FuncOp.from_py_func(f32, f64)
            def call_none(a, b):
                return none_return(a, b)

            ## Variants and optional feature tests.
            # CHECK-LABEL: func @from_name_arg
            @func.FuncOp.from_py_func(f32, f64, name="from_name_arg")
            def explicit_name(a, b):
                return b

            @func.FuncOp.from_py_func(f32, f64)
            def positional_func_op(a, b, func_op):
                assert isinstance(func_op, func.FuncOp)
                return b

            @func.FuncOp.from_py_func(f32, f64)
            def kw_func_op(a, b=None, func_op=None):
                assert isinstance(func_op, func.FuncOp)
                return b

            @func.FuncOp.from_py_func(f32, f64)
            def kwargs_func_op(a, b=None, **kwargs):
                assert isinstance(kwargs["func_op"], func.FuncOp)
                return b

            # CHECK-LABEL: func @explicit_results(%arg0: f32, %arg1: f64) -> f64
            # CHECK: return %arg1 : f64
            @func.FuncOp.from_py_func(f32, f64, results=[f64])
            def explicit_results(a, b):
                func.ReturnOp([b])

    print(m)


# CHECK-LABEL: TEST: testFromPyFuncErrors
@run
def testFromPyFuncErrors():
    with Context() as ctx, Location.unknown() as loc:
        m = builtin.ModuleOp()
        f32 = F32Type.get()
        f64 = F64Type.get()
        with InsertionPoint(m.body):
            try:

                @func.FuncOp.from_py_func(f64, results=[f64])
                def unary_return(a):
                    return a

            except AssertionError as e:
                # CHECK: Capturing a python function with explicit `results=` requires that the wrapped function returns None.
                print(e)


# CHECK-LABEL: TEST: testBuildFuncOp
@run
def testBuildFuncOp():
    ctx = Context()
    with Location.unknown(ctx) as loc:
        m = builtin.ModuleOp()

        f32 = F32Type.get()
        tensor_type = RankedTensorType.get((2, 3, 4), f32)
        with InsertionPoint.at_block_begin(m.body):
            f = func.FuncOp(
                name="some_func",
                type=FunctionType.get(
                    inputs=[tensor_type, tensor_type], results=[tensor_type]
                ),
                visibility="nested",
            )
            # CHECK: Name is: "some_func"
            print("Name is: ", f.name)

            # CHECK: Type is: (tensor<2x3x4xf32>, tensor<2x3x4xf32>) -> tensor<2x3x4xf32>
            print("Type is: ", f.type)

            # CHECK: Visibility is: "nested"
            print("Visibility is: ", f.visibility)

            try:
                entry_block = f.entry_block
            except IndexError as e:
                # CHECK: External function does not have a body
                print(e)

            with InsertionPoint(f.add_entry_block()):
                func.ReturnOp([f.entry_block.arguments[0]])
                pass

            try:
                f.add_entry_block()
            except IndexError as e:
                # CHECK: The function already has an entry block!
                print(e)

            # Try the callback builder and passing type as tuple.
            f = func.FuncOp(
                name="some_other_func",
                type=([tensor_type, tensor_type], [tensor_type]),
                visibility="nested",
                body_builder=lambda f: func.ReturnOp([f.entry_block.arguments[0]]),
            )

    # CHECK: module  {
    # CHECK:  func nested @some_func(%arg0: tensor<2x3x4xf32>, %arg1: tensor<2x3x4xf32>) -> tensor<2x3x4xf32> {
    # CHECK:   return %arg0 : tensor<2x3x4xf32>
    # CHECK:  }
    # CHECK:  func nested @some_other_func(%arg0: tensor<2x3x4xf32>, %arg1: tensor<2x3x4xf32>) -> tensor<2x3x4xf32> {
    # CHECK:   return %arg0 : tensor<2x3x4xf32>
    # CHECK:  }
    print(m)


# CHECK-LABEL: TEST: testFuncArgumentAccess
@run
def testFuncArgumentAccess():
    with Context() as ctx, Location.unknown():
        ctx.allow_unregistered_dialects = True
        module = Module.create()
        f32 = F32Type.get()
        f64 = F64Type.get()
        with InsertionPoint(module.body):
            f = func.FuncOp("some_func", ([f32, f32], [f32, f32]))
            with InsertionPoint(f.add_entry_block()):
                func.ReturnOp(f.arguments)
            f.arg_attrs = ArrayAttr.get(
                [
                    DictAttr.get(
                        {
                            "custom_dialect.foo": StringAttr.get("bar"),
                            "custom_dialect.baz": UnitAttr.get(),
                        }
                    ),
                    DictAttr.get({"custom_dialect.qux": ArrayAttr.get([])}),
                ]
            )
            f.result_attrs = ArrayAttr.get(
                [
                    DictAttr.get({"custom_dialect.res1": FloatAttr.get(f32, 42.0)}),
                    DictAttr.get({"custom_dialect.res2": FloatAttr.get(f64, 256.0)}),
                ]
            )

            other = func.FuncOp("other_func", ([f32, f32], []))
            with InsertionPoint(other.add_entry_block()):
                func.ReturnOp([])
            other.arg_attrs = [
                DictAttr.get({"custom_dialect.foo": StringAttr.get("qux")}),
                DictAttr.get(),
            ]

    # CHECK: [{custom_dialect.baz, custom_dialect.foo = "bar"}, {custom_dialect.qux = []}]
    print(f.arg_attrs)

    # CHECK: [{custom_dialect.res1 = 4.200000e+01 : f32}, {custom_dialect.res2 = 2.560000e+02 : f64}]
    print(f.result_attrs)

    # CHECK: func @some_func(
    # CHECK: %[[ARG0:.*]]: f32 {custom_dialect.baz, custom_dialect.foo = "bar"},
    # CHECK: %[[ARG1:.*]]: f32 {custom_dialect.qux = []}) ->
    # CHECK: f32 {custom_dialect.res1 = 4.200000e+01 : f32},
    # CHECK: f32 {custom_dialect.res2 = 2.560000e+02 : f64})
    # CHECK: return %[[ARG0]], %[[ARG1]] : f32, f32
    #
    # CHECK: func @other_func(
    # CHECK: %{{.*}}: f32 {custom_dialect.foo = "qux"},
    # CHECK: %{{.*}}: f32)
    print(module)


# CHECK-LABEL: testDenseElementsAttr
@run
def testDenseElementsAttr():
    with Context(), Location.unknown():
        values = np.arange(4, dtype=np.int32)
        i32 = IntegerType.get_signless(32)
        print(DenseElementsAttr.get(values, type=i32))
        # CHECK{LITERAL}: dense<[0, 1, 2, 3]> : tensor<4xi32>
        print(DenseElementsAttr.get(values, type=i32, shape=(2, 2)))
        # CHECK{LITERAL}: dense<[[0, 1], [2, 3]]> : tensor<2x2xi32>
        print(DenseElementsAttr.get(values, type=VectorType.get((2, 2), i32)))
        # CHECK{LITERAL}: dense<[[0, 1], [2, 3]]> : vector<2x2xi32>



# python3 createBenchmark.py MyRegexBenchmark
# reference: https://github.com/apple/swift/blob/main/benchmark/scripts/create_benchmark.py

import argparse
import os

template = """import _StringProcessing

extension BenchmarkRunner {{
  mutating func add{name}() {{
  }}
}}
"""

def main():
    p = argparse.ArgumentParser()
    p.add_argument("name", help="The name of the new benchmark to be created")
    args = p.parse_args()
    
    # create a file in Sources/RegexBenchmark/Suite with the benchmark template
    create_benchmark_file(args.name)
    
    # add to the registration function in BenchmarkRunner
    register_benchmark(args.name)

def create_benchmark_file(name):
    contents = template.format(name= name)
    relative_path = create_relative_path("../Sources/RegexBenchmark/Suite/")
    source_file_path = os.path.join(relative_path, name + ".swift")
    
    print(f"Creating new benchmark file: {source_file_path}")
    with open(source_file_path, "w") as f:
        f.write(contents)

def register_benchmark(name):
    relative_path = create_relative_path("../Sources/RegexBenchmark/BenchmarkRegistration.swift")

    # read current contents into an array
    file_contents = []
    with open(relative_path, "r") as f:
        file_contents = f.readlines()
    
    new_file_contents = []
    for line in file_contents:
        if "end of registrations" not in line:
            new_file_contents.append(line)
        else:
            # add the newest benchmark
            new_file_contents.append(f"    self.add{name}()\n")
            new_file_contents.append(line)
    
    # write the new contents
    with open(relative_path, "w") as f:
        for line in new_file_contents:
            f.write(line)
            
def create_relative_path(file_path):
    return os.path.join(os.path.dirname(__file__), file_path)

if __name__ == "__main__":
    main()



"""
Test lldb watchpoint that uses 'watchpoint set -w write -s size' to watch a pointed location with size.
"""


import lldb
from lldbsuite.test.decorators import *
from lldbsuite.test.lldbtest import *
from lldbsuite.test import lldbutil


class WatchLocationUsingWatchpointSetTestCase(TestBase):
    NO_DEBUG_INFO_TESTCASE = True

    # on arm64 targets, lldb has incorrect hit-count / ignore-counts
    # for watchpoints when they are hit with multiple threads at
    # the same time.  Tracked as llvm.org/pr49433
    # or rdar://93863107 inside Apple.
    def affected_by_radar_93863107(self):
        return (
            self.getArchitecture() in ["arm64", "arm64e"]
        ) and self.platformIsDarwin()

    def setUp(self):
        # Call super's setUp().
        TestBase.setUp(self)
        # Our simple source filename.
        self.source = "main.cpp"
        # Find the line number to break inside main().
        self.line = line_number(self.source, "// Set break point at this line.")
        # This is for verifying that watch location works.
        self.violating_func = "do_bad_thing_with_location"
        # Build dictionary to have unique executable names for each test
        # method.

    @skipIf(oslist=["linux"], archs=["aarch64", "arm"], bugnumber="llvm.org/pr26031")
    @skipIfWindows  # This test is flaky on Windows
    def test_watchlocation_using_watchpoint_set(self):
        """Test watching a location with 'watchpoint set expression -w write -s size' option."""
        self.build()
        self.setTearDownCleanup()

        exe = self.getBuildArtifact("a.out")
        target = self.dbg.CreateTarget(exe)

        # Add a breakpoint to set a watchpoint when stopped on the breakpoint.
        lldbutil.run_break_set_by_file_and_line(
            self, None, self.line, num_expected_locations=1
        )

        # Run the program.
        self.runCmd("run", RUN_SUCCEEDED)

        # We should be stopped again due to the breakpoint.
        # The stop reason of the thread should be breakpoint.
        self.expect(
            "thread list",
            STOPPED_DUE_TO_BREAKPOINT,
            substrs=["stopped", "stop reason = breakpoint"],
        )

        # Now let's set a write-type watchpoint pointed to by 'g_char_ptr' and
        # with offset as 7.
        # The main.cpp, by design, misbehaves by not following the agreed upon
        # protocol of only accessing the allowable index range of [0, 6].
        self.expect(
            "watchpoint set expression -w write -s 1 -- g_char_ptr + 7",
            WATCHPOINT_CREATED,
            substrs=["Watchpoint created", "size = 1", "type = w"],
        )
        self.runCmd("expr unsigned val = g_char_ptr[7]; val")
        self.expect(self.res.GetOutput().splitlines()[0], exe=False, endstr=" = 0")

        # Use the '-v' option to do verbose listing of the watchpoint.
        # The hit count should be 0 initially.
        self.expect("watchpoint list -v", substrs=["hit_count = 0"])

        # Check the underlying SBWatchpoint.
        watchpoint = target.GetWatchpointAtIndex(0)
        self.assertEqual(watchpoint.GetWatchSize(), 1)
        self.assertEqual(watchpoint.GetHitCount(), 0)
        self.assertEqual(watchpoint.GetWatchSpec(), "g_char_ptr + 7")

        self.runCmd("process continue")

        # We should be stopped again due to the watchpoint (write type), but
        # only once.  The stop reason of the thread should be watchpoint.
        self.expect(
            "thread list",
            STOPPED_DUE_TO_WATCHPOINT,
            substrs=[
                "stopped",
                self.violating_func,
                "stop reason = watchpoint",
            ],
        )

        # Switch to the thread stopped due to watchpoint and issue some
        # commands.
        self.switch_to_thread_with_stop_reason(lldb.eStopReasonWatchpoint)
        self.runCmd("thread backtrace")
        self.runCmd("expr unsigned val = g_char_ptr[7]; val")
        self.expect(self.res.GetOutput().splitlines()[0], exe=False, endstr=" = 99")

        # Use the '-v' option to do verbose listing of the watchpoint.
        # The hit count should now be the same as the number of threads that
        # stopped on a watchpoint.
        threads = lldbutil.get_stopped_threads(
            self.process(), lldb.eStopReasonWatchpoint
        )

        if not self.affected_by_radar_93863107():
            self.expect("watchpoint list -v", substrs=["hit_count = %d" % len(threads)])

        self.runCmd("thread backtrace all")



import lldb
from lldbsuite.test.lldbtest import *
from lldbsuite.test.decorators import *
import lldbsuite.test.lldbutil as lldbutil
import unittest2


class SwiftGenericClassInHashedContainerTest(TestBase):

    @swiftTest
    def test(self):
        """Tests that the type of hashed container whose value type is a non-generic class declared inside a generic class can be read from instance metadata"""
        self.build()
        lldbutil.run_to_source_breakpoint(self,"break here", lldb.SBFileSpec("main.swift"))
        self.expect("v val", substrs=["(a.A<Int>.B) val"])



"""
Test target commands: target.auto-install-main-executable.
"""

import socket
import time
import lldbgdbserverutils

from lldbsuite.test.decorators import *
from lldbsuite.test.lldbtest import *
from lldbsuite.test import lldbutil


class TestAutoInstallMainExecutable(TestBase):
    NO_DEBUG_INFO_TESTCASE = True

    @skipIfRemote
    @skipIfWindows  # This test is flaky on Windows
    def test_target_auto_install_main_executable(self):
        if lldbgdbserverutils.get_lldb_server_exe() is None:
            self.skipTest("lldb-server not found")
        self.build()

        hostname = socket.getaddrinfo("localhost", 0, proto=socket.IPPROTO_TCP)[0][4][0]
        listen_url = "[%s]:0" % hostname

        port_file = self.getBuildArtifact("port")
        commandline_args = [
            "platform",
            "--listen",
            listen_url,
            "--socket-file",
            port_file,
        ]
        self.spawnSubprocess(lldbgdbserverutils.get_lldb_server_exe(), commandline_args)

        socket_id = lldbutil.wait_for_file_on_target(self, port_file)

        new_platform = lldb.SBPlatform("remote-" + self.getPlatform())
        self.dbg.SetSelectedPlatform(new_platform)

        connect_url = "connect://[%s]:%s" % (hostname, socket_id)
        connect_opts = lldb.SBPlatformConnectOptions(connect_url)
        self.assertSuccess(new_platform.ConnectRemote(connect_opts))

        wd = self.getBuildArtifact("wd")
        os.mkdir(wd)
        new_platform.SetWorkingDirectory(wd)

        # Manually install the modified binary.
        src_device = lldb.SBFileSpec(self.getBuildArtifact("a.device.out"))
        dest = lldb.SBFileSpec(os.path.join(wd, "a.out"))
        self.assertSuccess(new_platform.Put(src_device, dest))

        # Test the default setting.
        self.expect(
            "settings show target.auto-install-main-executable",
            substrs=["target.auto-install-main-executable (boolean) = true"],
            msg="Default settings for target.auto-install-main-executable failed.",
        )

        # Disable the auto install.
        self.runCmd("settings set target.auto-install-main-executable false")
        self.expect(
            "settings show target.auto-install-main-executable",
            substrs=["target.auto-install-main-executable (boolean) = false"],
        )

        # Create the target with the original file.
        self.runCmd(
            "target create --remote-file %s %s "
            % (dest.fullpath, self.getBuildArtifact("a.out"))
        )

        self.expect("process launch", substrs=["exited with status = 74"])



import lldb
from lldbsuite.test.decorators import *
import lldbsuite.test.lldbtest as lldbtest
import lldbsuite.test.lldbutil as lldbutil


class TestSwiftArrayUninitialized(lldbtest.TestBase):
    @swiftTest
    def test(self):
        """Test unitialized global arrays"""
        self.build()
        lldbutil.run_to_source_breakpoint(
            self, 'break here', lldb.SBFileSpec('main.swift'))
        self.expect("target variable -- array_unused", substrs=['<uninitialized>'])
        self.expect("target variable -- array_used_empty", substrs=['0 values'])



"""Test lldb reloads the inferior after it was changed during the session."""


import time
import lldb
from lldbsuite.test.decorators import *
from lldbsuite.test.lldbtest import *
from lldbsuite.test import configuration
from lldbsuite.test import lldbutil


class ChangedInferiorTestCase(TestBase):
    @skipIf(hostoslist=["windows"])
    @no_debug_info_test
    def test_inferior_crashing(self):
        """Test lldb reloads the inferior after it was changed during the session."""
        self.build()
        self.inferior_crashing()
        self.cleanup()
        # lldb needs to recognize the inferior has changed. If lldb needs to check the
        # new module timestamp, make sure it is not the same as the old one, so add a
        # 1 second delay.
        time.sleep(1)
        d = {"C_SOURCES": "main2.c"}
        self.build(dictionary=d)
        self.setTearDownCleanup(dictionary=d)
        self.inferior_not_crashing()

    def setUp(self):
        # Call super's setUp().
        TestBase.setUp(self)
        # Find the line number of the crash.
        self.line1 = line_number("main.c", "// Crash here.")
        self.line2 = line_number("main2.c", "// Not crash here.")

    def inferior_crashing(self):
        """Inferior crashes upon launching; lldb should catch the event and stop."""
        self.exe = self.getBuildArtifact("a.out")
        self.runCmd("file " + self.exe, CURRENT_EXECUTABLE_SET)

        self.runCmd("run", RUN_SUCCEEDED)

        # We should have one crashing thread
        self.assertEqual(
            len(
                lldbutil.get_crashed_threads(
                    self, self.dbg.GetSelectedTarget().GetProcess()
                )
            ),
            1,
            STOPPED_DUE_TO_EXC_BAD_ACCESS,
        )

        # And it should report the correct line number.
        self.expect("thread backtrace all", substrs=["main.c:%d" % self.line1])

    def inferior_not_crashing(self):
        """Test lldb reloads the inferior after it was changed during the session."""
        self.runCmd("process kill")
        self.runCmd("run", RUN_SUCCEEDED)
        self.runCmd("process status")

        self.assertNotEqual(
            len(
                lldbutil.get_crashed_threads(
                    self, self.dbg.GetSelectedTarget().GetProcess()
                )
            ),
            1,
            "Inferior changed, but lldb did not perform a reload",
        )

        # Break inside the main.
        lldbutil.run_break_set_by_file_and_line(
            self, "main2.c", self.line2, num_expected_locations=1, loc_exact=True
        )

        self.runCmd("run", RUN_SUCCEEDED)

        # The stop reason of the thread should be breakpoint.
        self.expect(
            "thread list",
            STOPPED_DUE_TO_BREAKPOINT,
            substrs=["stopped", "stop reason = breakpoint"],
        )

        self.runCmd("frame variable int_ptr")
        self.expect("frame variable *int_ptr", substrs=["= 7"])
        self.expect("expression *int_ptr", substrs=["= 7"])



"""
Test the lldb platform Python API.
"""


import lldb
from lldbsuite.test.decorators import *
from lldbsuite.test.lldbtest import *
from lldbsuite.test import lldbutil


class PlatformPythonTestCase(TestBase):
    @add_test_categories(["pyapi"])
    @no_debug_info_test
    def test_platform_list(self):
        """Test SBDebugger::GetNumPlatforms() & GetPlatformAtIndex() API"""
        # Verify the host platform is present by default.
        initial_num_platforms = self.dbg.GetNumPlatforms()
        self.assertGreater(initial_num_platforms, 0)
        host_platform = self.dbg.GetPlatformAtIndex(0)
        self.assertTrue(
            host_platform.IsValid() and host_platform.GetName() == "host",
            "The host platform is present",
        )
        # Select another platform and verify that the platform is added to
        # the platform list.
        platform_idx = self.dbg.GetNumAvailablePlatforms() - 1
        if platform_idx < 1:
            self.fail("No platforms other than host are available")
        platform_data = self.dbg.GetAvailablePlatformInfoAtIndex(platform_idx)
        platform_name = platform_data.GetValueForKey("name").GetStringValue(100)
        self.assertNotEqual(platform_name, "host")
        self.dbg.SetCurrentPlatform(platform_name)
        selected_platform = self.dbg.GetSelectedPlatform()
        self.assertTrue(selected_platform.IsValid())
        self.assertEqual(selected_platform.GetName(), platform_name)
        self.assertEqual(self.dbg.GetNumPlatforms(), initial_num_platforms + 1)
        platform_found = False
        for platform_idx in range(self.dbg.GetNumPlatforms()):
            platform = self.dbg.GetPlatformAtIndex(platform_idx)
            if platform.GetName() == platform_name:
                platform_found = True
                break
        self.assertTrue(platform_found)

    @add_test_categories(["pyapi"])
    @no_debug_info_test
    def test_host_is_connected(self):
        # We've already tested that this one IS the host platform.
        host_platform = self.dbg.GetPlatformAtIndex(0)
        self.assertTrue(
            host_platform.IsConnected(), "The host platform is always connected"
        )

    @add_test_categories(["pyapi"])
    @no_debug_info_test
    def test_available_platform_list(self):
        """Test SBDebugger::GetNumAvailablePlatforms() and GetAvailablePlatformInfoAtIndex() API"""
        num_platforms = self.dbg.GetNumAvailablePlatforms()
        self.assertGreater(
            num_platforms, 0, "There should be at least one platform available"
        )

        for i in range(num_platforms):
            platform_data = self.dbg.GetAvailablePlatformInfoAtIndex(i)
            name_data = platform_data.GetValueForKey("name")
            desc_data = platform_data.GetValueForKey("description")
            self.assertTrue(name_data and name_data.IsValid(), "Platform has a name")
            self.assertEqual(
                name_data.GetType(),
                lldb.eStructuredDataTypeString,
                "Platform name is a string",
            )
            self.assertTrue(
                desc_data and desc_data.IsValid(), "Platform has a description"
            )
            self.assertEqual(
                desc_data.GetType(),
                lldb.eStructuredDataTypeString,
                "Platform description is a string",
            )

    @add_test_categories(["pyapi"])
    @no_debug_info_test
    @skipIfRemote
    def test_shell_interpreter(self):
        """Test a shell with a custom interpreter"""
        platform = self.dbg.GetSelectedPlatform()
        self.assertTrue(platform.IsValid())

        sh_cmd = lldb.SBPlatformShellCommand("/bin/zsh", "echo $0")
        self.assertIn("/bin/zsh", sh_cmd.GetShell())
        self.assertIn("echo $0", sh_cmd.GetCommand())

        self.build()
        sh_cmd.SetShell(self.getBuildArtifact("a.out"))
        err = platform.Run(sh_cmd)
        self.assertSuccess(err)
        self.assertIn("SUCCESS", sh_cmd.GetOutput())



"""
Describe the purpose of the test class here.
"""


import lldb
import lldbsuite.test.lldbutil as lldbutil
from lldbsuite.test.lldbtest import *


class RenameThisSampleTestTestCase(TestBase):
    # If your test case doesn't stress debug info, then
    # set this to true.  That way it won't be run once for
    # each debug info format.
    NO_DEBUG_INFO_TESTCASE = True

    def test_sample_rename_this(self):
        """There can be many tests in a test case - describe this test here."""
        self.build()
        self.main_source_file = lldb.SBFileSpec("main.c")
        self.sample_test()

    def setUp(self):
        # Call super's setUp().
        TestBase.setUp(self)
        # Set up your test case here. If your test doesn't need any set up then
        # remove this method from your TestCase class.

    def sample_test(self):
        """You might use the test implementation in several ways, say so here."""

        # This function starts a process, "a.out" by default, sets a source
        # breakpoint, runs to it, and returns the thread, process & target.
        # It optionally takes an SBLaunchOption argument if you want to pass
        # arguments or environment variables.
        (target, process, thread, bkpt) = lldbutil.run_to_source_breakpoint(
            self, "Set a breakpoint here", self.main_source_file
        )

        frame = thread.GetFrameAtIndex(0)
        test_var = frame.FindVariable("test_var")
        self.assertSuccess(test_var.GetError(), "Failed to fetch test_var")
        test_value = test_var.GetValueAsUnsigned()
        self.assertEqual(test_value, 10, "Got the right value for test_var")

    def sample_test_no_launch(self):
        """Same as above but doesn't launch a process."""

        target = self.createTestTarget()
        self.expect_expr("global_test_var", result_value="10")



# TestObjCIvarDiscovery.py
#
# This source file is part of the Swift.org open source project
#
# Copyright (c) 2014 - 2016 Apple Inc. and the Swift project authors
# Licensed under Apache License v2.0 with Runtime Library Exception
#
# See https://swift.org/LICENSE.txt for license information
# See https://swift.org/CONTRIBUTORS.txt for the list of Swift project authors
#
# ------------------------------------------------------------------------------
"""
Test that we can correctly see ivars from the Objective-C runtime
"""
import lldb
from lldbsuite.test.lldbtest import *
from lldbsuite.test.decorators import *
import lldbsuite.test.lldbutil as lldbutil
import os
import os.path
import re
import unittest2
import shutil

class TestObjCIVarDiscovery(TestBase):
    @skipUnlessDarwin
    @skipIf(debug_info=no_match("dsym"))
    @swiftTest
    def test_nodbg(self):
        self.build()
        shutil.rmtree(self.getBuildArtifact("aTestFramework.framework/Versions/A/aTestFramework.dSYM"))
        self.do_test(False)

    @skipUnlessDarwin
    @skipIf(debug_info=no_match("dsym"))
    @swiftTest
    def test_dbg(self):
        self.build()
        self.do_test(True)

    def prepare_value(self, value):
        value.SetPreferDynamicValue(lldb.eDynamicCanRunTarget)
        value.SetPreferSyntheticValue(True)
        return value

    def do_test(self, dbg):
        """Test that we can correctly see ivars from the Objective-C runtime"""
        if lldb.remote_platform:
            wd = lldb.remote_platform.GetWorkingDirectory()
            directory = 'aTestFramework.framework/Versions/A/'
            filename = directory + '/aTestFramework'
            cur_dir = wd
            for d in directory.split('/'):
                err = lldb.remote_platform.MakeDirectory(
                    os.path.join(cur_dir, d))
                self.assertFalse(err.Fail(), 'Failed to mkdir ' + d + ':' + str(err))
                cur_dir = os.path.join(cur_dir, d)
            err = lldb.remote_platform.Put(
                lldb.SBFileSpec(self.getBuildArtifact(filename)),
                lldb.SBFileSpec(os.path.join(wd, filename)))
            self.assertFalse(err.Fail(), 'Failed to copy ' + filename + ':' + str(err))

        # Launch the process, and do not stop at the entry point.
        lldbutil.run_to_source_breakpoint(
            self, 'Set breakpoint here', lldb.SBFileSpec('main.swift'),
        #    extra_images=['aTestFramework.framework/aTestFramework']
        )

        if dbg:
            self.expect(
                "image list", substrs=["Contents/Resources/DWARF/aTestFramework"])
        else:
            self.expect(
                "image list",
                substrs=["Contents/Resources/DWARF/aTestFramework"],
                matching=False)

        self.runCmd("frame variable -d run --show-types --ptr-depth=1")

        obj = self.prepare_value(self.frame().FindVariable("object"))

        mysubclass = self.prepare_value(obj.GetChildAtIndex(0))
        myclass = self.prepare_value(mysubclass.GetChildAtIndex(0))

        m_pair = myclass.GetChildMemberWithName("m_pair")
        m_pair_A = m_pair.GetChildMemberWithName("A")
        m_pair_B = m_pair.GetChildMemberWithName("B")

        self.assertEqual(m_pair_A.GetValueAsUnsigned(), 1)
        self.assertEqual(m_pair_B.GetValueAsUnsigned(), 2)

        m_derived = self.prepare_value(
            myclass.GetChildMemberWithName("m_base"))

        m_derivedX = m_derived.GetChildMemberWithName("m_DerivedX")

        self.assertEqual(m_derivedX.GetValueAsUnsigned(), 1)

        m_numbers = self.prepare_value(
            myclass.GetChildMemberWithName("m_myclass_numbers"))

        self.assertTrue(
            m_numbers.GetSummary() == '3 elements',
            "m_myclass_numbers != 3 elements")

        m_subclass_ivar = mysubclass.GetChildMemberWithName("m_subclass_ivar")
        self.assertTrue(
            m_subclass_ivar.GetValueAsUnsigned() == 42,
            "m_subclass_ivar != 42")

        m_mysubclass_s = mysubclass.GetChildMemberWithName("m_mysubclass_s")
        self.assertTrue(
            m_mysubclass_s.GetSummary() == '"an NSString here"',
            'm_subclass_s != "an NSString here"')

        swiftivar = obj.GetChildMemberWithName("swiftivar")
        self.assertTrue(
            swiftivar.GetSummary() == '"Hey Swift!"', "swiftivar != Hey Swift")

        silly = self.prepare_value(obj.GetChildMemberWithName("silly"))

        silly_x = silly.GetChildMemberWithName("x")
        silly_url = silly.GetChildMemberWithName("url")

        self.assertTrue(silly_x.GetValueAsUnsigned() == 12, "x != 12")
        self.assertTrue(
            silly_url.GetSummary() == '"http://www.apple.com"',
            "url != apple.com")



"""
Tests std::stack functionality.
"""

from lldbsuite.test.decorators import *
from lldbsuite.test.lldbtest import *
from lldbsuite.test import lldbutil


class TestStack(TestBase):
    @add_test_categories(["libc++"])
    @skipIf(compiler=no_match("clang"))
    @skipIfLinux  # Declaration in some Linux headers causes LLDB to crash.
    @skipIf(bugnumber="rdar://97622854")
    def test(self):
        self.build()

        lldbutil.run_to_source_breakpoint(
            self, "// Set break point at this line.", lldb.SBFileSpec("main.cpp")
        )

        self.runCmd("settings set target.import-std-module true")

        # Test std::stack functionality with a std::deque.
        stack_type = "std::stack<C>"
        size_type = stack_type + "::size_type"

        self.expect_expr("s_deque", result_type=stack_type)
        self.expect("expr s_deque.pop()")
        self.expect("expr s_deque.push({4})")
        self.expect_expr("s_deque.size()", result_type=size_type, result_value="3")
        self.expect_expr("s_deque.top().i", result_type="int", result_value="4")
        self.expect("expr s_deque.emplace(5)")
        self.expect_expr("s_deque.top().i", result_type="int", result_value="5")

        # Test std::stack functionality with a std::vector.
        stack_type = "std::stack<C, std::vector<C> >"
        size_type = stack_type + "::size_type"

        self.expect_expr("s_vector", result_type=stack_type)
        self.expect("expr s_vector.pop()")
        self.expect("expr s_vector.push({4})")
        self.expect_expr("s_vector.size()", result_type=size_type, result_value="3")
        self.expect_expr("s_vector.top().i", result_type="int", result_value="4")
        self.expect("expr s_vector.emplace(5)")
        self.expect_expr("s_vector.top().i", result_type="int", result_value="5")

        # Test std::stack functionality with a std::list.
        stack_type = "std::stack<C, std::list<C> >"
        size_type = stack_type + "::size_type"
        self.expect_expr("s_list", result_type=stack_type)
        self.expect("expr s_list.pop()")
        self.expect("expr s_list.push({4})")
        self.expect_expr("s_list.size()", result_type=size_type, result_value="3")
        self.expect_expr("s_list.top().i", result_type="int", result_value="4")
        self.expect("expr s_list.emplace(5)")
        self.expect_expr("s_list.top().i", result_type="int", result_value="5")



import lldb
from lldbsuite.test.lldbtest import *
from lldbsuite.test.decorators import *
import lldbsuite.test.lldbutil as lldbutil
import unittest2

class TestSwiftMissingVFSOverlay(TestBase):

    NO_DEBUG_INFO_TESTCASE = True
    
    # Don't run ClangImporter tests if Clangimporter is disabled.
    @skipIf(setting=('symbols.use-swift-clangimporter', 'false'))
    @skipUnlessDarwin
    @swiftTest
    def test(self):
        """This used to be a test for a diagnostic, however,
        this is no longer an unrecoverable error"""
        self.build()
        lldbutil.run_to_source_breakpoint(
            self, "break here", lldb.SBFileSpec("main.swift"),
            extra_images=["Foo"]
        )
        self.expect("expr y", substrs=["42"])



# TestSwiftieFormatting.py
#
# This source file is part of the Swift.org open source project
#
# Copyright (c) 2014 - 2016 Apple Inc. and the Swift project authors
# Licensed under Apache License v2.0 with Runtime Library Exception
#
# See https://swift.org/LICENSE.txt for license information
# See https://swift.org/CONTRIBUTORS.txt for the list of Swift project authors
#
# ------------------------------------------------------------------------------
"""
Test that data formatters honor Swift conventions
"""
import lldb
from lldbsuite.test.lldbtest import *
from lldbsuite.test.decorators import *
import lldbsuite.test.lldbutil as lldbutil
import os
import unittest2


class TestSwiftieFormatting(TestBase):
    @skipUnlessDarwin
    @swiftTest
    def test_swiftie_formatting(self):
        """Test that data formatters honor Swift conventions"""
        self.build()
        lldbutil.run_to_source_breakpoint(
            self, 'Set breakpoint here', lldb.SBFileSpec('main.swift'))

        swcla = self.frame().FindVariable("swcla")
        swcla.SetPreferDynamicValue(lldb.eDynamicCanRunTarget)
        swcla.SetPreferSyntheticValue(True)

        ns_a = swcla.GetChildMemberWithName("ns_a")
        self.assertTrue(
            ns_a.GetSummary() == '"Hello Swift"',
            "ns_a summary wrong")

        ns_d = swcla.GetChildMemberWithName("ns_d")
        self.assertTrue(ns_d.GetSummary() == '0 bytes', "ns_d summary wrong")

        IntWidth = 64
        if self.getArchitecture() in ['arm', 'armv7', 'armv7k', 'i386']:
            IntWidth = 32

        ns_n = swcla.GetChildMemberWithName("ns_n")
        self.assertTrue(ns_n.GetSummary() == ("Int%d(30)" % IntWidth), "ns_n summary wrong")

        ns_u = swcla.GetChildMemberWithName("ns_u")
        self.assertTrue(ns_u.GetSummary() == ('"page.html -- http://www.apple.com"'), "ns_u summary wrong")

        swcla = self.frame().EvaluateExpression("swcla")
        swcla.SetPreferDynamicValue(lldb.eDynamicCanRunTarget)
        swcla.SetPreferSyntheticValue(True)

        ns_a = swcla.GetChildMemberWithName("ns_a")
        self.assertTrue(
            ns_a.GetSummary() == '"Hello Swift"',
            "ns_a summary wrong")

        ns_d = swcla.GetChildMemberWithName("ns_d")
        self.assertTrue(ns_d.GetSummary() == '0 bytes', "ns_d summary wrong")

        ns_n = swcla.GetChildMemberWithName("ns_n")
        self.assertTrue(ns_n.GetSummary() == ("Int%d(30)" % IntWidth), "ns_n summary wrong")

        ns_u = swcla.GetChildMemberWithName("ns_u")
        self.assertTrue(ns_u.GetSummary() == ('"page.html -- http://www.apple.com"'), "ns_u summary wrong")

        nsarr = self.frame().FindVariable("nsarr")
        nsarr.SetPreferDynamicValue(lldb.eDynamicCanRunTarget)
        nsarr.SetPreferSyntheticValue(True)

        nsarr0 = nsarr.GetChildAtIndex(0)
        nsarr0.SetPreferDynamicValue(lldb.eDynamicCanRunTarget)
        nsarr0.SetPreferSyntheticValue(True)
        nsarr1 = nsarr.GetChildAtIndex(1)
        nsarr1.SetPreferDynamicValue(lldb.eDynamicCanRunTarget)
        nsarr1.SetPreferSyntheticValue(True)
        nsarr3 = nsarr.GetChildAtIndex(3)
        nsarr3.SetPreferDynamicValue(lldb.eDynamicCanRunTarget)
        nsarr3.SetPreferSyntheticValue(True)

        self.assertTrue(
            nsarr0.GetSummary() == ("Int%d(2)" % IntWidth),
            'nsarr[0] summary wrong')
        self.assertTrue(
            nsarr1.GetSummary() == ("Int%d(3)" % IntWidth),
            'nsarr[1] summary wrong')
        self.assertTrue(
            nsarr3.GetSummary() == ("Int%d(5)" % IntWidth),
            'nsarr[3] summary wrong')

        self.expect(
            'frame variable -d run nsarr[4] --ptr-depth=1',
            substrs=[
                '"One"',
                '"Two"',
                '"Three"'])
        self.expect(
            'frame variable -d run nsarr[5] --ptr-depth=1',
            substrs=[
                ("Int%d(1)" % IntWidth),
                ("Int%d(2)" % IntWidth),
                ("Int%d(3)" % IntWidth)])



#!/usr/bin/python3

from git import Repo
import re
import sys


def get_version_from_tag(tag):
    m = re.match("llvmorg-([0-9]+)\.([0-9]+)\.([0-9]+)(-rc[0-9]+)?$", tag)
    if m:
        if m.lastindex == 4:
            # We have an rc tag.
            return m.group(1, 2, 3)
        # We have a final release tag.
        return (m.group(1), m.group(2), str(int(m.group(3)) + 1))

    m = re.match("llvmorg-([0-9]+)-init", tag)
    if m:
        return (m.group(1), "0", "0")

    raise Exception(f"error: Tag is not valid: {tag}")


version = sys.argv[1]

repo = Repo()

tag = repo.git.describe(tags=True, abbrev=0)
expected_version = ".".join(get_version_from_tag(tag))

if version != expected_version:
    print("error: Expected version", expected_version, "but found version", version)
    sys.exit(1)

print("Versions match:", version, expected_version)
sys.exit(0)



import lldb
from lldbsuite.test.decorators import *
from lldbsuite.test.lldbtest import *
from lldbsuite.test import lldbutil


class TestObjcPoHint(TestBase):
    NO_DEBUG_INFO_TESTCASE = True

    def test_show_po_hint(self):
        ### Test that the po hint is shown once with the DWIM print command
        self.build()
        _, _, _, _ = lldbutil.run_to_source_breakpoint(
            self, "Set breakpoint here", lldb.SBFileSpec("main.m")
        )
        # Make sure the hint is printed the first time
        self.expect(
            "dwim-print -O -- foo",
            substrs=[
                "note: object description requested, but type doesn't implement "
                'a custom object description. Consider using "p" instead of "po"',
                "<Foo: 0x",
            ],
        )

        # Make sure it's not printed again.
        self.expect(
            "dwim-print -O -- foo",
            substrs=[
                "note: object description"
            ],
            matching=False,
        )

    def test_show_po_hint_disabled(self):
        ### Test that when the setting is disabled the hint is not printed
        self.build()
        _, _, _, _ = lldbutil.run_to_source_breakpoint(
            self, "Set breakpoint here", lldb.SBFileSpec("main.m")
        )
        self.runCmd("setting set show-dont-use-po-hint false")
        # Make sure the hint is printed the first time
        self.expect(
            "dwim-print -O -- foo",
            substrs=[
                "note: object description"
            ],
            matching=False,
        )



# Part of the LLVM Project, under the Apache License v2.0 with LLVM Exceptions.
# See https://llvm.org/LICENSE.txt for license information.
# SPDX-License-Identifier: Apache-2.0 WITH LLVM-exception

from ipykernel.kernelapp import IPKernelApp
from .kernel import MlirOptKernel

IPKernelApp.launch_instance(kernel_class=MlirOptKernel)



"""Test that a global ObjC object found before the process is started updates correctly."""


import lldb
from lldbsuite.test.decorators import *
from lldbsuite.test.lldbtest import *
from lldbsuite.test import lldbutil


class TestObjCGlobalVar(TestBase):
    def setUp(self):
        # Call super's setUp().
        TestBase.setUp(self)
        self.main_source = lldb.SBFileSpec("main.m")

    @add_test_categories(["pyapi"])
    def test_with_python_api(self):
        """Test that a global ObjC object found before the process is started updates correctly."""
        self.build()
        exe = self.getBuildArtifact("a.out")

        target = self.dbg.CreateTarget(exe)
        self.assertTrue(target, VALID_TARGET)

        bkpt = target.BreakpointCreateBySourceRegex("NSLog", self.main_source)
        self.assertTrue(bkpt, VALID_BREAKPOINT)

        # Before we launch, make an SBValue for our global object pointer:
        g_obj_ptr = target.FindFirstGlobalVariable("g_obj_ptr")
        self.assertSuccess(g_obj_ptr.GetError(), "Made the g_obj_ptr")
        self.assertEqual(
            g_obj_ptr.GetValueAsUnsigned(10), 0, "g_obj_ptr is initially null"
        )

        # Now launch the process, and do not stop at entry point.
        process = target.LaunchSimple(None, None, self.get_process_working_directory())

        self.assertTrue(process, PROCESS_IS_VALID)

        # The stop reason of the thread should be breakpoint.
        threads = lldbutil.get_threads_stopped_at_breakpoint(process, bkpt)
        if len(threads) != 1:
            self.fail("Failed to stop at breakpoint 1.")

        thread = threads[0]

        dyn_value = g_obj_ptr.GetDynamicValue(lldb.eDynamicCanRunTarget)
        self.assertTrue(dyn_value.GetError().Success(), "Dynamic value is valid")
        self.assertEqual(dyn_value.GetObjectDescription(), "Some NSString")



# TestSwiftDynamicTypeResolutionImportConflict.py
#
# This source file is part of the Swift.org open source project
#
# Copyright (c) 2018 Apple Inc. and the Swift project authors
# Licensed under Apache License v2.0 with Runtime Library Exception
#
# See https://swift.org/LICENSE.txt for license information
# See https://swift.org/CONTRIBUTORS.txt for the list of Swift project authors
#
# ------------------------------------------------------------------------------

import lldb
from lldbsuite.test.lldbtest import *
from lldbsuite.test.decorators import *
import lldbsuite.test.lldbutil as lldbutil
import os
import unittest2
import shutil

class TestSwiftDynamicTypeResolutionImportConflict(TestBase):
    # Don't run ClangImporter tests if Clangimporter is disabled.
    @skipIf(setting=('symbols.use-swift-clangimporter', 'false'))
    @skipUnlessDarwin
    @swiftTest
    def test(self):
        """
        This testcase causes the scratch context to get destroyed by a
        conflict that is triggered via dynamic type resolution. The
        conflict is triggered by "frame variable" alone. The final
        "expr" command is just there to test that after "fr var" has
        destroyed the scratch context we can recover.

        """
        # To ensure we hit the rebuild problem remove the cache to avoid caching.
        mod_cache = self.getBuildArtifact("my-clang-modules-cache")
        if os.path.isdir(mod_cache):
          shutil.rmtree(mod_cache)

        self.runCmd('settings set symbols.clang-modules-cache-path "%s"'
                    % mod_cache)
        self.build()
        target, _, _, _ = lldbutil.run_to_source_breakpoint(self, "break here",
                                          lldb.SBFileSpec('main.swift'),
                                          extra_images=['Dylib', 'Conflict'])
        # Destroy the scratch context with a dynamic clang type lookup.
        self.expect("target var -d run-target -- foofoo",
                    substrs=['(Conflict.C) foofoo'])
        self.expect("target var -- foofoo",
                    substrs=['(Conflict.C) foofoo'])
        dylib_breakpoint = target.BreakpointCreateBySourceRegex(
            'break here', lldb.SBFileSpec('Dylib.swift'))
        self.assertGreater(dylib_breakpoint.GetNumLocations(), 0)

        self.expect("continue")
        self.expect("bt", substrs=['Dylib.swift'])
        self.expect("fr v -d no-dynamic-values -- input",
                    substrs=['(Dylib.LibraryProtocol) input'])

        # Because we are now in a per-module scratch context, we don't
        # expect dynamic type resolution to find a type defined inside
        # the other dylib.
        self.expect("fr v -d run-target -- input",
                    substrs=['(a.FromMainModule) input', "i = 1"])
        # FIXME: The output here is nondeterministic.
        #self.expect("expr -d run-target -- input",
        #            substrs=['(a.FromMainModule)'])



#!/usr/bin/env python

# ---------------------------------------------------------------------
# Be sure to add the python path that points to the LLDB shared library.
#
# # To use this in the embedded python interpreter using "lldb" just
# import it with the full path using the "command script import"
# command
#   (lldb) command script import /path/to/cmdtemplate.py
# ---------------------------------------------------------------------

import inspect
import lldb
import argparse
import shlex
import sys

# Each new breakpoint gets a unique ID starting from 1.
nextid = 1
# List of breakpoint set from python, the key is the ID and the value the
# actual breakpoint. These are NOT LLDB SBBreakpoint objects.
breakpoints = dict()

exprOptions = lldb.SBExpressionOptions()
exprOptions.SetIgnoreBreakpoints()
exprOptions.SetLanguage(lldb.eLanguageTypeC)


class MlirDebug:
    """MLIR debugger commands
    This is the class that hooks into LLDB and registers the `mlir` command.
    Other providers can register subcommands below this one.
    """

    lldb_command = "mlir"
    parser = None

    def __init__(self, debugger, unused):
        super().__init__()
        self.create_options()
        self.help_string = MlirDebug.parser.format_help()

    @classmethod
    def create_options(cls):
        if MlirDebug.parser:
            return MlirDebug.parser
        usage = "usage: %s [options]" % (cls.lldb_command)
        description = "TODO."

        # Pass add_help_option = False, since this keeps the command in line
        # with lldb commands, and we wire up "help command" to work by
        # providing the long & short help methods below.
        MlirDebug.parser = argparse.ArgumentParser(
            prog=cls.lldb_command, usage=usage, description=description, add_help=False
        )
        MlirDebug.subparsers = MlirDebug.parser.add_subparsers(dest="command")
        return MlirDebug.parser

    def get_short_help(self):
        return "MLIR debugger commands"

    def get_long_help(self):
        return self.help_string

    def __call__(self, debugger, command, exe_ctx, result):
        # Use the Shell Lexer to properly parse up command options just like a
        # shell would
        command_args = shlex.split(command)

        try:
            args = MlirDebug.parser.parse_args(command_args)
        except:
            result.SetError("option parsing failed")
            raise
        args.func(args, debugger, command, exe_ctx, result)

    @classmethod
    def on_process_start(frame, bp_loc, dict):
        print("Process started")


class SetControl:
    # Define the subcommands that controls what to do when a breakpoint is hit.
    # The key is the subcommand name, the value is a tuple of the command ID to
    # pass to MLIR and the help string.
    commands = {
        "apply": (1, "Apply the current action and continue the execution"),
        "skip": (2, "Skip the current action and continue the execution"),
        "step": (3, "Step into the current action"),
        "next": (4, "Step over the current action"),
        "finish": (5, "Step out of the current action"),
    }

    @classmethod
    def register_mlir_subparser(cls):
        for cmd, (cmdInt, help) in cls.commands.items():
            parser = MlirDebug.subparsers.add_parser(
                cmd,
                help=help,
            )
            parser.set_defaults(func=cls.process_options)

    @classmethod
    def process_options(cls, options, debugger, command, exe_ctx, result):
        frame = exe_ctx.GetFrame()
        if not frame.IsValid():
            result.SetError("No valid frame (program not running?)")
            return
        cmdInt = cls.commands.get(options.command, None)
        if not cmdInt:
            result.SetError("Invalid command: %s" % (options.command))
            return

        result = frame.EvaluateExpression(
            "((bool (*)(int))mlirDebuggerSetControl)(%d)" % (cmdInt[0]),
            exprOptions,
        )
        if not result.error.Success():
            print("Error setting up command: %s" % (result.error))
            return
        debugger.SetAsync(True)
        result = exe_ctx.GetProcess().Continue()
        debugger.SetAsync(False)


class PrintContext:
    @classmethod
    def register_mlir_subparser(cls):
        cls.parser = MlirDebug.subparsers.add_parser(
            "context", help="Print the current context"
        )
        cls.parser.set_defaults(func=cls.process_options)

    @classmethod
    def process_options(cls, options, debugger, command, exe_ctx, result):
        frame = exe_ctx.GetFrame()
        if not frame.IsValid():
            result.SetError("Can't print context without a valid frame")
            return
        result = frame.EvaluateExpression(
            "((bool (*)())&mlirDebuggerPrintContext)()", exprOptions
        )
        if not result.error.Success():
            print("Error printing context: %s" % (result.error))
            return


class Backtrace:
    @classmethod
    def register_mlir_subparser(cls):
        cls.parser = MlirDebug.subparsers.add_parser(
            "backtrace", aliases=["bt"], help="Print the current backtrace"
        )
        cls.parser.set_defaults(func=cls.process_options)
        cls.parser.add_argument("--context", default=False, action="store_true")

    @classmethod
    def process_options(cls, options, debugger, command, exe_ctx, result):
        frame = exe_ctx.GetFrame()
        if not frame.IsValid():
            result.SetError(
                "Can't backtrace without a valid frame (program not running?)"
            )
        result = frame.EvaluateExpression(
            "((bool(*)(bool))mlirDebuggerPrintActionBacktrace)(%d)" % (options.context),
            exprOptions,
        )
        if not result.error.Success():
            print("Error printing breakpoints: %s" % (result.error))
            return


###############################################################################
# Cursor manipulation
###############################################################################


class PrintCursor:
    @classmethod
    def register_mlir_subparser(cls):
        cls.parser = MlirDebug.subparsers.add_parser(
            "cursor-print", aliases=["cursor-p"], help="Print the current cursor"
        )
        cls.parser.add_argument(
            "--print-region", "--regions", "-r", default=False, action="store_true"
        )
        cls.parser.set_defaults(func=cls.process_options)

    @classmethod
    def process_options(cls, options, debugger, command, exe_ctx, result):
        frame = exe_ctx.GetFrame()
        if not frame.IsValid():
            result.SetError(
                "Can't print cursor without a valid frame (program not running?)"
            )
        result = frame.EvaluateExpression(
            "((bool(*)(bool))mlirDebuggerCursorPrint)(%d)" % (options.print_region),
            exprOptions,
        )
        if not result.error.Success():
            print("Error printing cursor: %s" % (result.error))
            return


class SelectCursorFromContext:
    @classmethod
    def register_mlir_subparser(cls):
        cls.parser = MlirDebug.subparsers.add_parser(
            "cursor-select-from-context",
            aliases=["cursor-s"],
            help="Select the cursor from the current context",
        )
        cls.parser.add_argument("index", type=int, help="Index in the context")
        cls.parser.set_defaults(func=cls.process_options)

    @classmethod
    def process_options(cls, options, debugger, command, exe_ctx, result):
        frame = exe_ctx.GetFrame()
        if not frame.IsValid():
            result.SetError(
                "Can't manipulate cursor without a valid frame (program not running?)"
            )
        result = frame.EvaluateExpression(
            "((bool(*)(int))mlirDebuggerCursorSelectIRUnitFromContext)(%d)"
            % options.index,
            exprOptions,
        )
        if not result.error.Success():
            print("Error manipulating cursor: %s" % (result.error))
            return


class CursorSelectParent:
    @classmethod
    def register_mlir_subparser(cls):
        cls.parser = MlirDebug.subparsers.add_parser(
            "cursor-parent", aliases=["cursor-up"], help="Select the cursor parent"
        )
        cls.parser.set_defaults(func=cls.process_options)

    @classmethod
    def process_options(cls, options, debugger, command, exe_ctx, result):
        frame = exe_ctx.GetFrame()
        if not frame.IsValid():
            result.SetError(
                "Can't manipulate cursor without a valid frame (program not running?)"
            )
        result = frame.EvaluateExpression(
            "((bool(*)())mlirDebuggerCursorSelectParentIRUnit)()",
            exprOptions,
        )
        if not result.error.Success():
            print("Error manipulating cursor: %s" % (result.error))
            return


class SelectCursorChild:
    @classmethod
    def register_mlir_subparser(cls):
        cls.parser = MlirDebug.subparsers.add_parser(
            "cursor-child", aliases=["cursor-c"], help="Select the nth child"
        )
        cls.parser.add_argument("index", type=int, help="Index of the child to select")
        cls.parser.set_defaults(func=cls.process_options)

    @classmethod
    def process_options(cls, options, debugger, command, exe_ctx, result):
        frame = exe_ctx.GetFrame()
        if not frame.IsValid():
            result.SetError(
                "Can't manipulate cursor without a valid frame (program not running?)"
            )
        result = frame.EvaluateExpression(
            "((bool(*)(int))mlirDebuggerCursorSelectChildIRUnit)(%d)" % options.index,
            exprOptions,
        )
        if not result.error.Success():
            print("Error manipulating cursor: %s" % (result.error))
            return


class CursorSelecPrevious:
    @classmethod
    def register_mlir_subparser(cls):
        cls.parser = MlirDebug.subparsers.add_parser(
            "cursor-previous",
            aliases=["cursor-prev"],
            help="Select the cursor previous element",
        )
        cls.parser.set_defaults(func=cls.process_options)

    @classmethod
    def process_options(cls, options, debugger, command, exe_ctx, result):
        frame = exe_ctx.GetFrame()
        if not frame.IsValid():
            result.SetError(
                "Can't manipulate cursor without a valid frame (program not running?)"
            )
        result = frame.EvaluateExpression(
            "((bool(*)())mlirDebuggerCursorSelectPreviousIRUnit)()",
            exprOptions,
        )
        if not result.error.Success():
            print("Error manipulating cursor: %s" % (result.error))
            return


class CursorSelecNext:
    @classmethod
    def register_mlir_subparser(cls):
        cls.parser = MlirDebug.subparsers.add_parser(
            "cursor-next", aliases=["cursor-n"], help="Select the cursor next element"
        )
        cls.parser.set_defaults(func=cls.process_options)

    @classmethod
    def process_options(cls, options, debugger, command, exe_ctx, result):
        frame = exe_ctx.GetFrame()
        if not frame.IsValid():
            result.SetError(
                "Can't manipulate cursor without a valid frame (program not running?)"
            )
        result = frame.EvaluateExpression(
            "((bool(*)())mlirDebuggerCursorSelectNextIRUnit)()",
            exprOptions,
        )
        if not result.error.Success():
            print("Error manipulating cursor: %s" % (result.error))
            return


###############################################################################
# Breakpoints
###############################################################################


class EnableBreakpoint:
    @classmethod
    def register_mlir_subparser(cls):
        cls.parser = MlirDebug.subparsers.add_parser(
            "enable", help="Enable a single breakpoint (given its ID)"
        )
        cls.parser.add_argument("id", help="ID of the breakpoint to enable")
        cls.parser.set_defaults(func=cls.process_options)

    @classmethod
    def process_options(cls, options, debugger, command, exe_ctx, result):
        bp = breakpoints.get(int(options.id), None)
        if not bp:
            result.SetError("No breakpoint with ID %d" % int(options.id))
            return
        bp.enable(exe_ctx.GetFrame())


class DisableBreakpoint:
    @classmethod
    def register_mlir_subparser(cls):
        cls.parser = MlirDebug.subparsers.add_parser(
            "disable", help="Disable a single breakpoint (given its ID)"
        )
        cls.parser.add_argument("id", help="ID of the breakpoint to disable")
        cls.parser.set_defaults(func=cls.process_options)

    @classmethod
    def process_options(cls, options, debugger, command, exe_ctx, result):
        bp = breakpoints.get(int(options.id), None)
        if not bp:
            result.SetError("No breakpoint with ID %s" % options.id)
            return
        bp.disable(exe_ctx.GetFrame())


class ListBreakpoints:
    @classmethod
    def register_mlir_subparser(cls):
        cls.parser = MlirDebug.subparsers.add_parser(
            "list", help="List all current breakpoints"
        )
        cls.parser.set_defaults(func=cls.process_options)

    @classmethod
    def process_options(cls, options, debugger, command, exe_ctx, result):
        for id, bp in sorted(breakpoints.items()):
            print(id, type(id), str(bp), "enabled" if bp.isEnabled else "disabled")


class Breakpoint:
    def __init__(self):
        global nextid
        self.id = nextid
        nextid += 1
        breakpoints[self.id] = self
        self.isEnabled = True

    def enable(self, frame=None):
        self.isEnabled = True
        if not frame or not frame.IsValid():
            return
        # use a C cast to force the type of the breakpoint handle to be void * so
        # that we don't rely on DWARF. Also add a fake bool return value otherwise
        # LLDB can't signal any error with the expression evaluation (at least I don't know how).
        cmd = (
            "((bool (*)(void *))mlirDebuggerEnableBreakpoint)((void *)%s)" % self.handle
        )
        result = frame.EvaluateExpression(cmd, exprOptions)
        if not result.error.Success():
            print("Error enabling breakpoint: %s" % (result.error))
            return

    def disable(self, frame=None):
        self.isEnabled = False
        if not frame or not frame.IsValid():
            return
        # use a C cast to force the type of the breakpoint handle to be void * so
        # that we don't rely on DWARF. Also add a fake bool return value otherwise
        # LLDB can't signal any error with the expression evaluation (at least I don't know how).
        cmd = (
            "((bool (*)(void *)) mlirDebuggerDisableBreakpoint)((void *)%s)"
            % self.handle
        )
        result = frame.EvaluateExpression(cmd, exprOptions)
        if not result.error.Success():
            print("Error disabling breakpoint: %s" % (result.error))
            return


class TagBreakpoint(Breakpoint):
    mlir_subcommand = "break-on-tag"

    def __init__(self, tag):
        super().__init__()
        self.tag = tag

    def __str__(self):
        return "[%d] TagBreakpoint(%s)" % (self.id, self.tag)

    @classmethod
    def register_mlir_subparser(cls):
        cls.parser = MlirDebug.subparsers.add_parser(
            cls.mlir_subcommand, help="add a breakpoint on actions' tag matching"
        )
        cls.parser.set_defaults(func=cls.process_options)
        cls.parser.add_argument("tag", help="tag to match")

    @classmethod
    def process_options(cls, options, debugger, command, exe_ctx, result):
        breakpoint = TagBreakpoint(options.tag)
        print("Added breakpoint %s" % str(breakpoint))

        frame = exe_ctx.GetFrame()
        if frame.IsValid():
            breakpoint.install(frame)

    def install(self, frame):
        result = frame.EvaluateExpression(
            '((void *(*)(const char *))mlirDebuggerAddTagBreakpoint)("%s")'
            % (self.tag),
            exprOptions,
        )
        if not result.error.Success():
            print("Error installing breakpoint: %s" % (result.error))
            return
        # Save the handle, this is necessary to implement enable/disable.
        self.handle = result.GetValue()


class FileLineBreakpoint(Breakpoint):
    mlir_subcommand = "break-on-file"

    def __init__(self, file, line, col):
        super().__init__()
        self.file = file
        self.line = line
        self.col = col

    def __str__(self):
        return "[%d] FileLineBreakpoint(%s, %d, %d)" % (
            self.id,
            self.file,
            self.line,
            self.col,
        )

    @classmethod
    def register_mlir_subparser(cls):
        cls.parser = MlirDebug.subparsers.add_parser(
            cls.mlir_subcommand,
            help="add a breakpoint that filters on location of the IR affected by an action. The syntax is file:line:col where file and col are optional",
        )
        cls.parser.set_defaults(func=cls.process_options)
        cls.parser.add_argument("location", type=str)

    @classmethod
    def process_options(cls, options, debugger, command, exe_ctx, result):
        split_loc = options.location.split(":")
        file = split_loc[0]
        line = int(split_loc[1]) if len(split_loc) > 1 else -1
        col = int(split_loc[2]) if len(split_loc) > 2 else -1
        breakpoint = FileLineBreakpoint(file, line, col)
        print("Added breakpoint %s" % str(breakpoint))

        frame = exe_ctx.GetFrame()
        if frame.IsValid():
            breakpoint.install(frame)

    def install(self, frame):
        result = frame.EvaluateExpression(
            '((void *(*)(const char *, int, int))mlirDebuggerAddFileLineColLocBreakpoint)("%s", %d, %d)'
            % (self.file, self.line, self.col),
            exprOptions,
        )
        if not result.error.Success():
            print("Error installing breakpoint: %s" % (result.error))
            return
        # Save the handle, this is necessary to implement enable/disable.
        self.handle = result.GetValue()


def on_start(frame, bpno, err):
    print("MLIR debugger attaching...")
    for _, bp in sorted(breakpoints.items()):
        if bp.isEnabled:
            print("Installing breakpoint %s" % (str(bp)))
            bp.install(frame)
        else:
            print("Skipping disabled breakpoint %s" % (str(bp)))

    return True


def __lldb_init_module(debugger, dict):
    target = debugger.GetTargetAtIndex(0)
    debugger.SetAsync(False)
    if not target:
        print("No target is loaded, please load a target before loading this script.")
        return
    if debugger.GetNumTargets() > 1:
        print(
            "Multiple targets (%s) loaded, attaching MLIR debugging to %s"
            % (debugger.GetNumTargets(), target)
        )

    # Register all classes that have a register_lldb_command method
    module_name = __name__
    parser = MlirDebug.create_options()
    MlirDebug.__doc__ = parser.format_help()

    # Add the MLIR entry point to LLDB as a command.
    command = "command script add -o -c %s.%s %s" % (
        module_name,
        MlirDebug.__name__,
        MlirDebug.lldb_command,
    )
    debugger.HandleCommand(command)

    main_bp = target.BreakpointCreateByName("main")
    main_bp.SetScriptCallbackFunction("action_debugging.on_start")
    main_bp.SetAutoContinue(auto_continue=True)

    on_breackpoint = target.BreakpointCreateByName("mlirDebuggerBreakpointHook")

    print(
        'The "{0}" command has been installed for target `{1}`, type "help {0}" or "{0} '
        '--help" for detailed help.'.format(MlirDebug.lldb_command, target)
    )
    for _name, cls in inspect.getmembers(sys.modules[module_name]):
        if inspect.isclass(cls) and getattr(cls, "register_mlir_subparser", None):
            cls.register_mlir_subparser()
